
{.$define Debug}

program Solution;
{$M 16384,0,655360}

{$ifndef Debug}
{$A+,B-,D+,E-,F-,G+,I+,L+,N+,O-,P-,Q-,R-,S-,T-,V-,X+,Y+}
const
  inFile			= 'input.txt';
  outFile			= 'output.txt';
{$else}
{$A+,B-,D+,E-,F-,G+,I+,L+,N+,O-,P-,Q+,R+,S+,T-,V-,X+,Y+}
const
  inFile			= 'input.txt';
  outFile			= 'con';
  dbgFile			= 'con';
var
  dbg                           : Text;
{$endif}

const
  msgNoSol		 	= 'NO SOLUTIONS';
  eps				= 1e-7;

type
  float				= extended;

var
  xp,yp				: array[1..8] of float;
  x,y,z				: array[1..8] of float;
  ok				: boolean;

procedure WriteData; forward;
procedure Finalize; forward;

procedure Check;
label next;
var
  i,j				: integer;
  used				: array[1..8] of boolean;

begin
  fillchar( used, sizeof(used), false );
  for i := 1 to 8 do begin
    for j := 1 to 8 do
      if (abs( x[i] - xp[j] ) < eps) and
         (abs( y[i] - yp[j] ) < eps) and
	 not used[j] then begin
        used[j] := true;
        goto next;
      end;
    exit;
    next :
  end;
  ok := true;
  WriteData;
  Finalize;
end;

procedure Try( i, j, k : integer );
var
  x1,y1,x2,y2,x3,y3		: float;
  a,b				: float;
  d				: float;
  t1,t2				: float;


  procedure BuildCube( z1, z2, z3, x6, y6, z6 : float );
  begin
    x[1] := x1;
    y[1] := y1;
    z[1] := z1;
    x[2] := x2;
    y[2] := y2;
    z[2] := z2;
    x[3] := x3;
    y[3] := y3;
    z[3] := z3;
    x[6] := x6;
    y[6] := y6;
    z[6] := z6;
    x[7] := x[6] + x[3] - x[2];
    y[7] := y[6] + y[3] - y[2];
    z[7] := z[6] + z[3] - z[2];
    x[5] := x[6] + x[1] - x[2];
    y[5] := y[6] + y[1] - y[2];
    z[5] := z[6] + z[1] - z[2];
    x[4] := x[1] + x[3] - x[2];
    y[4] := y[1] + y[3] - y[2];
    z[4] := z[1] + z[3] - z[2];
    x[8] := x[5] + x[3] - x[2];
    y[8] := y[5] + y[3] - y[2];
    z[8] := z[5] + z[3] - z[2];
    Check;
  end;

  procedure Build( z1, z3 : float );
  var
    l				: float;
    ux,uy,uz			: float;
    vx,vy,vz			: float;
    wx,wy,wz			: float;

  begin
    l := sqrt( sqr(x1 - x2) + sqr(y1 - y2) + sqr( z1 ) );
    if abs( l - round( l ) ) > 1e-3 then exit;
    if abs( l ) < eps then exit;
    ux := (x1 - x2) / l;
    uy := (y1 - y2) / l;
    uz := z1 / l;
    vx := (x3 - x2) / l;
    vy := (y3 - y2) / l;
    vz := z3 / l;

    wx := uy * vz - vy * uz;
    wy := vx * uz - ux * vz;
    wz := ux * vy - vx * uy;

    wx := wx * l;
    wy := wy * l;
    wz := wz * l;
    BuildCube( z1, 0, z3, x2 + wx, y2 + wy, wz );
    BuildCube( z1, 0, z3, x2 - wx, y2 - wy, -wz );
  end;

  procedure Process( z1 : float );
  begin
    Build( z1, a / z1 );
  end;

begin
  if (i=2) and (j=1) and (k=3) then
  readln;
  x1 := xp[i];
  y1 := yp[i];
  x2 := xp[j];
  y2 := yp[j];
  x3 := xp[k];
  y3 := yp[k];
  a := - (x1 - x2) * (x3 - x2) - (y1 - y2) * (y3 - y2);
  b := sqr( x3 - x2 ) + sqr( y3 - y2 ) - sqr( x1 - x2 ) - sqr( y1 - y2 );
  if abs( b ) < eps then b := 0;
  if abs( a ) < eps then begin
    if b < eps then begin
      Build( 0, sqrt( -b ) );
      Build( 0, -sqrt( -b ) );
    end else begin
      Build( sqrt( b ), 0 );
      Build( -sqrt( b ), 0 );
    end;
    exit;
  end;
  d := sqr( b ) + 4 * sqr( a );
  if abs( d ) < eps then d := 0;
  if d < -eps then exit;
  d := sqrt( d );
  t1 := (b + d) / 2;
  t2 := (b - d) / 2;
  if abs( t1 ) < eps then t1 := 0;
  if abs( t2 ) < eps then t2 := 0;
  if t1 > eps/2 then begin
    Process( sqrt( t1 ) );
    Process( -sqrt( t1 ) );
  end;
  if t2 > eps/2 then begin
    Process( sqrt( t2 ) );
    Process( -sqrt( t2 ) );
  end;
end;

procedure Solve;
var
  i,j,k				: integer;

begin
  ok := false;
  for i := 1 to 8 do
    for j := 1 to 8 do
      for k := 1 to 8 do
        if (i <> j) and (j <> k) and (i <> k) then Try( i, j, k );
end;

procedure ReadData;
var
  i				: integer;

begin
  for i := 1 to 8 do read( xp[i], yp[i] );
end;

procedure WriteData;
var
  i				: integer;

begin
  if ok then
    for i := 1 to 8 do writeln( x[i]:0:10, ' ', y[i]:0:10, ' ', z[i]:0:10 )
  else
    writeln( msgNoSol );
end;

procedure Initialize;
begin

  assign( input, inFile );
  reset( input );

  assign( output, outFile );
  rewrite( output );

  {$ifdef Debug}
  assign( dbg, dbgFile );
  rewrite( dbg );
  writeln( '----Initialized----' );
  {$endif}

end;

procedure Finalize;
begin

  {$ifdef Debug}
  close( dbg );
  writeln( '----Finalized----' );
  {$endif}

  close( input );
  close( output );
  halt( 0 );

end;

begin
  Initialize;
  ReadData;
  Solve;
  WriteData;
  Finalize;
end.