
{.$define Debug}

program Solution;
{$M 16384,0,655360}

{$ifndef Debug}
{$A+,B-,D+,E-,F-,G+,I+,L+,N+,O-,P-,Q-,R-,S-,T-,V-,X+,Y+}
const
  inFile			= 'input.txt';
  outFile			= 'output.txt';
{$else}
{$A+,B-,D+,E-,F-,G+,I+,L+,N+,O-,P-,Q+,R+,S+,T-,V-,X+,Y+}
const
  inFile			= 'input.txt';
  outFile			= 'con';
  dbgFile			= 'con';
var
  dbg                           : Text;
{$endif}

const
  maxN				= 14;
  eps				= 1e-6;
  inf				= maxlongint;

type
  int				= longint;
  float				= extended;
  TBool				= array[1..maxN] of boolean;

var
  n				: integer;
  extra				: float;
  answer			: TBool;
  x,y,v,l			: array[1..maxN] of int;
  dist				: array[1..maxN,1..maxN] of float;
  dist2				: array[1..maxN,1..maxN] of int;
  bestUse			: TBool;
  bestCard			: integer;
  bestV				: int;
  bestExtra			: float;
  use				: TBool;

function Cardinal( var b : TBool ) : integer;
var
  i,j				: integer;

begin
  j := 0;
  for i := 1 to n do
    if b[i] then inc( j );
  Cardinal := j;
end;

function BuildConvexHull : float;
var
  i,j,k				: integer;
  minX,minY			: int;
  maxD				: int;
  newD				: int;
  len				: float;
  used				: TBool;
  start				: integer;
  v				: int;

begin
  len := 0;
  minX := inf;
  minY := inf;
  for i := 1 to n do
    if use[i] then
      if (y[i] < minY) or ((y[i] = minY) and (x[i] < minX)) then begin
        minX := x[i];
        minY := y[i];
        j := i;
      end;

  fillchar( used, sizeof(used), false );
  used[j] := true;
  start := j;
  i := j;

  repeat

    maxD := 0;
    k := i;
    for j := 1 to n do
      if use[j] and (not used[j] or (j = start)) then begin
        newD := sqr( x[i] - x[j] ) + sqr( y[i] - y[j] );
        v := (x[j] - x[k]) * (y[i] - y[k]) - (y[j] - y[k]) * (x[i] - x[k]);
        if (v < 0) or ((v = 0) and (newD > maxD)) then begin
          k := j;
          maxD := newD;
        end;
      end;

    if k = start then begin
      len := len + dist[i,start];
      BuildConvexHull := len;
      exit;
    end;

    len := len + dist[i,k];
    i := k;
    used[k] := true;

  until false;
end;

procedure Solve;
var
  xx				: word;
  i,j				: integer;
  curV				: int;
  curLen			: int;
  curCard			: integer;
  curExtra			: float;

begin
  for i := 1 to n do
    for j := 1 to n do begin
      dist2[i,j] := sqr(x[i] - x[j]) + sqr(y[i] - y[j]);
      dist[i,j] := sqrt( dist2[i,j] );
    end;

  bestV := inf;
  bestCard := maxint;

  for xx := 1 to (1 shl n) - 1 do begin

    fillchar( use, sizeof(use), false );
    for i := 1 to n do
      if xx and (1 shl (i-1)) <> 0 then use[i] := true;

    curV := 0;
    for i := 1 to n do
      if not use[i] then inc( curV, v[i] );
    if curV > bestV then continue;

    curCard := Cardinal( use );

    curLen := 0;
    for i := 1 to n do
      if not use[i] then inc( curLen, l[i] );
    curExtra := curLen - BuildConvexHull;
    if curExtra < -eps then continue;

    if (curV < bestV) or ((curV = bestV) and (curCard > bestCard)) then begin
      bestUse := use;
      bestV := curV;
      bestExtra := curExtra;
      bestCard := curCard;
    end;

  end;

  answer := bestUse;
  extra := bestExtra;

end;

procedure ReadData;
var
  i				: integer;

begin
  read( n );
  for i := 1 to n do read( x[i], y[i], v[i], l[i] );
end;

procedure WriteData;
var
  i				: integer;

begin
  write( 'Cut these trees:' );
  for i := 1 to n do
    if not answer[i] then write( ' ', i );
  writeln;
  writeln( 'Extra wood: ', extra:0:2 );
end;

procedure Initialize;
begin

  assign( input, inFile );
  reset( input );

  assign( output, outFile );
  rewrite( output );

  {$ifdef Debug}
  assign( dbg, dbgFile );
  rewrite( dbg );
  writeln( '----Initialized----' );
  {$endif}

end;

procedure Finalize;
begin

  {$ifdef Debug}
  close( dbg );
  writeln( '----Finalized----' );
  {$endif}

  close( input );
  close( output );
  halt( 0 );

end;

begin
  Initialize;
  ReadData;
  Solve;
  WriteData;
  Finalize;
end.
