
{.$define Debug}

program Solution;
{$M 16384,0,655360}

{$ifndef Debug}
{$A+,B-,D+,E-,F-,G+,I+,L+,N+,O-,P-,Q-,R-,S-,T-,V-,X+,Y+}
const
  inFile			= 'input.txt';
  outFile			= 'output.txt';
{$else}
{$A+,B-,D+,E-,F-,G+,I+,L+,N+,O-,P-,Q+,R+,S+,T-,V-,X+,Y+}
const
  inFile			= 'input.txt';
  outFile			= 'con';
  dbgFile			= 'con';
var
  dbg                           : Text;
{$endif}

const
  maxA                          = 26;
  maxR                          = 100;
  maxE                          = 10;
  inf                           = maxlongint;

type
  int                           = longint;

  TSet                          = set of 'A'..'Z';

  TQueue                        = object
    s                           : TSet;
    constructor                 Init;
    function                    Retrieve : char;
    procedure                   Insert( ch : char );
  end;

  TObject                       = object
    reqTime                     : int;
    procedure                   Process; virtual;
    procedure                   Schedule( t : int );
  end;

  TAgent                        = object (TObject)
    prevRoom                    : integer;
    newStage                    : integer;
    entered                     : boolean;
    state                       : ( _prep, _entering, _exiting, _dead,
                                    _in_cab, _to_cab, _waiting_cab,
                                    _to_elev, _waiting_elev, _in_elev );
    entTime                     : int;
    prevTime                    : int;
    exists                      : boolean;
    curTask                     : integer;
    nTasks                      : integer;
    stage                       : integer;
    signal                      : boolean;
    r                           : array[1..maxR] of integer;
    t                           : array[1..maxR] of int;
    constructor                 Init;
    procedure                   Process; virtual;
  end;

  TElevator                     = object (TObject)
    q                           : TQueue;
    constructor                 Init;
    procedure                   Process; virtual;
  end;

  TRoom                         = object (TObject)
    exists                      : boolean;
    q                           : TQueue;
    used                        : boolean;
    constructor                 Init;
    procedure                   Process; virtual;
  end;

var
  agents                        : array['A'..'Z'] of TAgent;
  rooms                         : array[0..maxR] of TRoom;
  elevs                         : array[0..maxE] of TElevator;
  active                        : char;
  minTime                       : int;
  curTime                       : int;
  curAgent                      : char;
  startTime                     : int;

function GetTime( s : string ) : int;
var
  hh,mm,ss                      : int;

begin
  if length( s ) <> 8 then s := '0' + s;
  hh := ord( s[2] ) - ord( '0' ) + 10 * ( ord( s[1] ) - ord( '0' ) );
  mm := ord( s[5] ) - ord( '0' ) + 10 * ( ord( s[4] ) - ord( '0' ) );
  ss := ord( s[8] ) - ord( '0' ) + 10 * ( ord( s[7] ) - ord( '0' ) );
  GetTime := ss + 60 * (mm + 60 * hh);
end;

procedure PrintTime( t : int );
var
  hh,mm,ss                      : int;

begin
  ss := t mod 60;
  mm := (t div 60) mod 60;
  hh := t div 3600;
  write( hh div 10, hh mod 10, ':',
         mm div 10, mm mod 10, ':',
         ss div 10, ss mod 10 );
end;

procedure PrintRoom( r : integer);
var
  a, b                          : integer;

begin
  a := r div 10 + 1;
  b := r mod 10 + 1;
  write( 'room ', a div 10, a mod 10,
         b div 10, b mod 10 );
end;

constructor TQueue.Init;
begin
  s := [];
end;

procedure TQueue.Insert( ch : char );
begin
  include( s, ch );
end;

function TQueue.Retrieve : char;
var
  i                             : char;

begin
  for i := 'A' to 'Z' do
    if i in s then begin
      exclude( s, i );
      Retrieve := i;
      exit;
    end;
  Retrieve := #0;
end;

procedure TObject.Process;
begin
end;


constructor TAgent.Init;
begin
end;

constructor TRoom.Init;
begin
end;

constructor TElevator.Init;
begin
end;

procedure TObject.Schedule( t : int );
begin
  reqTime := t;
  if t < minTime then minTime := t;
end;

procedure TAgent.Process;

  procedure EnterRoom;
  begin
    rooms[r[curTask]].q.Insert( curAgent );
    state := _waiting_cab;
    prevTime := curTime;
    Schedule( inf );
  end;

  procedure ProcessTask;
  begin
    if curTask > nTasks then begin
      if stage = 0 then begin
        prevTime := curTime;
        state := _exiting;
        Schedule( curTime + 30 );
        exit;
      end else begin
        prevRoom := r[nTasks];
        state := _to_elev;
        prevTime := curTime;
        newStage := 0;
        Schedule( curTime + 10 );
        exit;
      end;
    end;
    if stage = r[curTask] div 10 then begin
      if not entered then begin
        state := _to_cab;
        prevTime := curTime;
        Schedule( curTime + 10 );
      end else
        EnterRoom;
    end else begin
      newStage := r[curTask] div 10;
      if entered then begin
        state := _waiting_elev;
        prevTime := curTime;
        elevs[0].q.Insert( curAgent );
        Schedule( inf );
      end else begin
        prevRoom := r[curTask-1];
        state := _to_elev;
        prevTime := curTime;
        Schedule( curTime + 10 );
      end;
    end;
  end;


begin
  case state of

    _prep : begin
      if curTime = entTime then begin
        state := _entering;
        Schedule( entTime + 30 );
        exit;
      end;
      Schedule( entTime );
    end;

    _entering : begin
      if curTime = reqTime then begin
        if active = curAgent then begin
          PrintTime( entTime );
          write( ' ' );
          PrintTime( curTime );
          write( ' ' );
          writeln( 'Entry' );
        end;
        prevTime := curTime;
        stage := 0;
        curTask := 1;
        entered := true;
        ProcessTask;
      end else
        Schedule( entTime + 30 );

    end;

    _exiting : begin
      if curTime = reqTime then begin
        if active = curAgent then begin
          PrintTime( prevTime );
          write( ' ' );
          PrintTime( curTime );
          write( ' ' );
          writeln( 'Exit' );
        end;
        state := _dead;
      end else
        Schedule( prevTime + 30 );
    end;

    _to_cab : begin
      if curTime = reqTime then begin
        if (active = curAgent) and
           not entered and (curTime > prevTime) then begin
          PrintTime( prevTime );
          write( ' ' );
          PrintTime( curTime );
          write( ' ' );
          write( 'Transfer from ' );
          if prevRoom = -1 then
            write( 'elevator' )
          else
            PrintRoom( prevRoom );
          write( ' to ' );
          PrintRoom( r[curTask] );
          writeln;
        end;
        EnterRoom;
      end else
        Schedule( prevTime + 10 );
    end;

    _waiting_cab : if signal then begin
      entered := false;
      signal := false;
      if (curAgent = active) and (curTime > prevTime ) then begin
        PrintTime( prevTime );
        write( ' ' );
        PrintTime( curTime );
        write( ' ' );
        write( 'Waiting in front of ' );
        PrintRoom( r[curTask] );
        writeln;
      end;
      state := _in_cab;
      prevTime := curTime;
      rooms[r[curTask]].used := true;
      Schedule( curTime + t[curTask] );
    end;

    _in_cab : begin
      if curTime = reqTime then begin
        if active = curAgent then begin
          PrintTime( prevTime );
          write( ' ' );
          PrintTime( curTime );
          write( ' ' );
          write( 'Stay in ' );
          PrintRoom( r[curTask] );
          writeln;
        end;
        prevRoom := r[curTask];
        rooms[r[curTask]].used := false;
        inc( curTask );
        ProcessTask;
      end else
        Schedule( reqTime );
    end;

    _to_elev : begin
      if curTime = reqTime then begin
        if (active = curAgent) and not entered and
           (curTime > prevTime) then begin
          PrintTime( prevTime );
          write( ' ' );
          PrintTime( curTime );
          write( ' ' );
          write( 'Transfer from ' );
          PrintRoom( prevRoom );
          writeln( ' to elevator' );
        end;
        state := _waiting_elev;
        prevTime := curTime;
        elevs[stage].q.Insert( curAgent );
        Schedule( inf );
      end else
        Schedule( reqTime );
    end;

    _waiting_elev : if signal then begin
      signal := false;
      if (curAgent = active) and (curTime > prevTime ) then begin
        PrintTime( prevTime );
        write( ' ' );
        PrintTime( curTime );
        write( ' ' );
        writeln( 'Waiting in elevator queue' );
      end;
      state := _in_elev;
      prevTime := curTime;
      Schedule( curTime + abs(stage - newStage) * 30 );
      Schedule( reqTime );
    end;

    _in_elev : begin
      if curTime = reqTime then begin
        if active = curAgent then begin
          PrintTime( prevTime );
          write( ' ' );
          PrintTime( curTime );
          write( ' ' );
          writeln( 'Stay in elevator' );
        end;
        stage := newStage;
        if curTask > nTasks then begin
          ProcessTask;
          exit;
        end;
        prevRoom := -1;
        state := _to_cab;
        entered := false;
        prevTime := curTime;
        Schedule( curTime + 10 );
      end else
        Schedule( reqTime );
    end;

  end;

end;

procedure TElevator.Process;
var
  i                             : char;

begin
  if curTime <> reqTime then begin
    Schedule( reqTime );
    exit;
  end;
  if curtime = 35 then
  readln;
  i := q.Retrieve;
  if i <> #0 then
  agents[i].signal := true;
  Schedule( curTime + 5 );
end;

procedure TRoom.Process;
var
  i                             : char;

begin
  if not used and (q.s <> []) then begin
    i := q.Retrieve;
    agents[i].signal := true;
    Schedule( curTime );
    exit;
  end;
end;

procedure InitAll;
var
  i                             : char;
  j                             : integer;

begin
  for i := 'A' to 'Z' do begin
    agents[i].state := _prep;
    agents[i].signal := false;
  end;
  for j := 0 to maxR do begin
    rooms[j].q.Init;
  end;
  for j := 0 to maxE do begin
    elevs[j].q.Init;
    elevs[j].reqTime := startTime;
  end;
end;

procedure Simulate;
label try;
var
  i                             : char;
  j                             : integer;

begin

  curTime := starttime;
  repeat
    for i := 'A' to 'Z' do
      if agents[i].exists and (agents[i].state <> _dead) then goto try;
    exit;
    try :

    minTime := inf;

    for i := 'A' to 'Z' do
      if agents[i].exists then begin
        curAgent := i;
        agents[i].Process;
      end;

    for j := 0 to maxR do
      if rooms[j].exists then rooms[j].Process;
    for j := 0 to maxE do elevs[j].Process;

    for i := 'A' to 'Z' do
      if agents[i].exists and
         (agents[i].state in [ _waiting_elev, _waiting_cab] ) then begin
        curAgent := i;
        agents[i].Process;
      end;

    if minTime <> inf then curTime := minTime;
  until false;
end;

procedure Solve;
var
  ch                            : char;

begin
  startTime := inf;
  for ch := 'A' to 'Z' do
    if agents[ch].exists and (agents[ch].entTime < startTime) then
      startTime := agents[ch].entTime;

  startTime := startTime - startTime mod 5;
  for ch := 'A' to 'Z' do
    if agents[ch].exists then begin
      active := ch;
      writeln( ch );
      InitAll;
      Simulate;
      writeln;
    end;
end;

function strtok( var s : string ) : string;
var
  i                             : integer;

begin
  while s[1] = ' ' do delete( s, 1, 1 );
  i := pos( ' ', s );
  if i = 0 then begin
    strtok := s;
    s := '';
  end else begin
    strtok := copy( s, 1, i - 1 );
    delete( s, 1, i );
  end;
end;

procedure ReadData;
var
  s                             : string;
  r                             : string;
  ch                            : char;
  x                             : integer;

begin
  fillchar( agents, sizeof(agents), 0 );
  fillchar( elevs, sizeof(elevs), 0 );
  fillchar( rooms, sizeof(rooms), 0 );
  for x := 0 to maxE do elevs[x].Init;
  repeat
    readln( s );
    if s = '.' then break;
    r := strtok( s );
    ch := r[1];
    agents[ch].exists := true;
    agents[ch].entTime := GetTime( strtok( s ) );
    agents[ch].nTasks := 0;
    agents[ch].Init;
    repeat
      read( x );
      if x = 0 then begin
        readln;
        break;
      end;
      x := (x div 100 - 1) * 10 + x mod 100 - 1;
      inc( agents[ch].nTasks );
      agents[ch].r[agents[ch].nTasks] := x;
      rooms[x].exists := true;
      rooms[x].Init;
      readln( x );
      agents[ch].t[agents[ch].nTasks] := x;
    until false;
  until false;
end;

procedure Initialize;
begin

  assign( input, inFile );
  reset( input );

  assign( output, outFile );
  rewrite( output );

  {$ifdef Debug}
  assign( dbg, dbgFile );
  rewrite( dbg );
  writeln( '----Initialized----' );
  {$endif}

end;

procedure Finalize;
begin

  {$ifdef Debug}
  close( dbg );
  writeln( '----Finalized----' );
  {$endif}

  close( input );
  close( output );
  halt( 0 );

end;

begin
  Initialize;
  ReadData;
  Solve;
  Finalize;
end.
