
{.$define Debug}

program Solution;
{$M 16384,204800,204800}

{$ifndef Debug}
{$A+,B-,D+,E+,F-,G+,I+,L+,N+,O-,P-,Q-,R-,S-,T-,V-,X+,Y+}
const
  inFile			= 'input.txt';
  outFile			= 'output.txt';
{$else}
{$A+,B-,D+,E+,F-,G+,I+,L+,N+,O-,P-,Q+,R+,S+,T-,V-,X+,Y+}
const
  inFile			= 'input.txt';
  outFile			= 'con';
  dbgFile			= 'con';
var
  dbg                           : Text;
{$endif}

const
  alpha				= ['','','','','','','',
				   '','','','','','','','','',
				   '','','','','','','','','','',
				   '','','','','',''];

  maxET                         = 5;
  maxS                          = 255;
  maxQ                          = 5000;
  msgNoSol                      = 'NO SOLUTION';
  maxL                          = 80;

type

  TTag                          = ( ltQuest, ltAster, ltLetter,
                                    ltSet, ltLoRep, ltLoHiRep );

  TSSet                         = set of 1..maxS;
  TSet                          = set of char;

  TLexem                        = record
    tag                         : TTag;
    ch                          : char;
    s                           : TSet;
    lo,hi                       : integer;
  end;

  TState                        = object
    nET                         : integer;
    et                          : array[1..maxET] of integer;
    ref                         : integer;
    next                        : integer;
    prev                        : integer;
    nTrans                      : integer;

    constructor                 Init;
    procedure                   AppendET( id : integer );

  end;

  TAutomat                      = object
    nStates                     : integer;
    s                           : array[1..maxS] of TState;
    lexs                        : array[1..maxL] of TLexem;
    constructor                 Build( expr : string );
    procedure                   GetTrans( id, q : integer; var target : integer; var chSet : TSet );
  end;

  TQueue                        = array[1..maxQ] of record
    sa,sb                       : integer;
    prev                        : integer;
    ch                          : char;
  end;

  PQueue                        = ^TQueue;

  TMap                          = array[1..maxS,1..maxS] of boolean;
  PMap                          = ^TMap;

var
  a,b                           : string;
  autA,autB                     : TAutomat;
  q                             : PQueue;
  head,tail                     : integer;
  map                           : PMap;
  ok                            : boolean;
  answer                        : string;

function getid( a : char ) : integer;
const
  x                             : string = '';

var
  i                             : integer;

begin
  for i := 1 to 32 do
    if x[i] = a then begin
      getid := i;
      exit;
    end;
end;

function _upcase( ch : char ) : char;
const
  a                             : string = '㪥뢠஫ᬨ';
  b                             : string = '';

var
  i                             : integer;

begin
  for i := 1 to 32 do
    if a[i] = ch then begin
      _upcase := b[i];
      exit;
    end;
  _upcase := ch;
end;

procedure InitQueue;
begin
  head := 0;
  tail := 1;
  fillchar( map^, sizeof(map^), false );
end;

function ParseInt( var p : integer; s : string ) : integer;
var
  x                             : integer;
  code                          : integer;
  i                             : integer;

begin
  i := p;
  while s[p] in ['0'..'9'] do inc( p );
  val( copy( s, i, p - i ), x, code );
  ParseInt := x;
end;

function Parse( var lex : TLexem; var p : integer; s : string ) : boolean;
var
  i				: integer;
  ch				: char;
  intA,intB			: integer;
  chA,chB			: char;
  neg				: boolean;

begin
  s[length(s)+1] := #0;
  if p > length( s ) then begin
    Parse := false;
    exit;
  end;
  Parse := false;
  if _upcase( s[p] ) in alpha then begin
    lex.tag := ltLetter;
    lex.ch := _upcase( s[p] );
    inc( p );
    exit;
  end;
  if s[p] = '?' then begin
    lex.tag := ltQuest;
    inc( p );
    exit;
  end;
  if s[p] = '*' then begin
    lex.tag := ltAster;
    inc( p );
    exit;
  end;
  if s[p] = '@' then begin
    lex.tag := ltLoHiRep;
    lex.lo := 1;
    lex.hi := 1;
    inc( p );
    exit;
  end;
  if s[p] = '[' then begin
    lex.tag := ltSet;
    lex.s := [];
    inc( p );
    neg := false;
    if s[p] = '!' then begin
      neg := true;
      inc( p );
    end;
    while s[p] <> ']' do begin
      chA := _upcase( s[p] );
      inc( p );
      if (s[p] = ',') or (s[p] = ']') then begin
        include( lex.s, chA );
        continue;
      end;
      inc( p );
      chB := _upcase( s[p] );
      for ch := #0 to #255 do
        if (ch in alpha) and (getid( ch ) >= getid( chA )) and
           (getid( ch ) <= getid( chB )) then include( lex.s, ch );
      inc( p );
      if s[p] = ',' then inc( p );
    end;
    if neg then lex.s := alpha - lex.s;
    inc( p );
    exit;
  end;
  if s[p] = '{' then begin
    inc( p );
    intA := ParseInt( p, s );
    if s[p] = '}' then begin
      inc( p );
      lex.tag := ltLoHiRep;
      lex.lo := intA;
      lex.hi := intA;
      exit;
    end;
    inc( p );
    if s[p] = '}' then begin
      inc( p );
      lex.tag := ltLoRep;
      lex.lo := intA;
      exit;
    end;
    intB := ParseInt( p, s );
    inc( p );
    lex.tag := ltLoHiRep;
    lex.lo := intA;
    lex.hi := intB;
    exit;
  end;
end;

procedure ProcessET( var aut : TAutomat; k : integer; var s : TSSet );
var
  i                             : integer;

begin
  if k in s then exit;
  include( s, k );
  for i := 1 to aut.s[k].nET do ProcessET( aut, aut.s[k].et[i], s );
end;

procedure PushQueue( sa, sb, prev : integer; ch : char );
var
  xa,xb                         : TSSet;
  i,j                           : integer;

  procedure IPushQueue( sa, sb : integer );
  begin
    if map^[sa,sb] then exit;
    map^[sa,sb] := true;
    inc( head );
    q^[head].sa := sa;
    q^[head].sb := sb;
    q^[head].prev := prev;
    q^[head].ch := ch;
  end;

begin
  xa := [];
  ProcessET( autA, sa, xa );
  xb := [];
  ProcessET( autB, sb, xb );
  for i := 1 to autA.nStates + 1 do
    if i in xa then
      for j := 1 to autB.nStates + 1 do
        if j in xb then IPushQueue( i, j );
end;

procedure PopQueue( var sa, sb, prev : integer; var ch : char );
begin
  sa := q^[tail].sa;
  sb := q^[tail].sb;
  prev := q^[tail].prev;
  ch := q^[tail].ch;
  inc( tail );
end;

function Retrieve( var ch : TSet ) : char;
const
  x                             : string = '';

var
  i                             : integer;

begin
  for i := 1 to 32 do
    if x[i] in ch then begin
      Retrieve := x[i];
      exit;
    end;
end;

constructor TState.Init;
begin
  nET := 0;
end;

procedure TState.AppendET( id : integer );
begin
  inc( nET );
  et[nET] := id;
end;

constructor TAutomat.Build( expr : string );
var
  nLex                          : integer;
  p                             : integer;
  lex                           : TLexem;
  prev                          : integer;
  i                             : integer;

begin
  nLex := 0;
  nStates := 0;
  p := 1;
  while p <= length( expr ) do begin
    Parse( lex, p, expr );
    inc( nLex );
    lexs[nLex] := lex;
    inc( nStates );
    s[nStates].ref := nLex;
    case lex.tag of
      ltQuest : begin
        s[nStates].next := nStates + 1;
        s[nStates].nTrans := 1;
        prev := nStates;
      end;
      ltAster : begin
        s[nStates].next := nStates + 1;
        s[nStates].nTrans := 1;
        s[nStates].AppendET( nStates + 1 );
        prev := nStates;
      end;
      ltSet : begin
        s[nStates].next := nStates + 1;
        s[nStates].nTrans := 1;
        prev := nStates;
      end;
      ltLetter : begin
        s[nStates].next := nStates + 1;
        s[nStates].nTrans := 1;
        prev := nStates;
      end;
      ltLoHiRep : begin
        if (lex.lo <> 1) or (lex.hi <> 1) then continue;
        s[nStates].next := nStates + 1;
        s[nStates].nTrans := 0;
        s[nStates].AppendET( prev );
        s[nStates].AppendET( nStates + 1 );
        prev := nStates;
      end;
    end;
  end;
end;

procedure TAutomat.GetTrans( id, q : integer; var target : integer;
                             var chSet : TSet );
begin
  case lexs[s[id].ref].tag of
    ltQuest : begin
      chSet := alpha;
      target := s[id].next;
    end;
    ltAster : begin
      chSet := alpha;
      target := id;
    end;
    ltSet : begin
      chSet := lexs[s[id].ref].s;
      target := s[id].next;
    end;
    ltLetter : begin
      chSet := [ lexs[s[id].ref].ch ];
      target := s[id].next;
    end;
    ltLoHiRep : begin
    end;
  end;
end;

procedure WriteData; forward;
procedure Finalize; forward;

procedure Solve;
var
  sa,sb                         : integer;
  cha,chb                       : TSet;
  i,j                           : integer;
  newa,newb                     : integer;
  prev                          : integer;
  ch                            : char;

begin
  autA.Build( a );
  autB.Build( b );
  InitQueue;
  PushQueue( 1, 1, 0, #0 );
  while head >= tail do begin

    PopQueue( sa, sb, prev, ch );

    if (sa > autA.nStates) and
       (sb > autB.nStates) then begin
      answer := '';
      i := prev;
      while i <> 0 do begin
        answer := ch + answer;
        ch := q^[i].ch;
        i := q^[i].prev;
      end;
      ok := true;
      WriteData;
      Finalize;
    end;

    if (sa > autA.nStates) or
       (sb > autB.nStates) then continue;

    for i := 1 to autA.s[sa].nTrans do begin
      autA.GetTrans( sa, i, newa, cha );
      for j := 1 to autB.s[sb].nTrans do begin
        autB.GetTrans( sb, i, newb, chb );
        chb := chb * cha;
        if chb <> [] then begin
          ch := Retrieve( chb );
          PushQueue( newa, newb, tail - 1, ch );
        end;
      end;
    end;

  end;
  ok := false;
end;

procedure ReadData;
begin
  readln( a );
  readln( b );
end;

procedure WriteData;
begin
  if ok then
     writeln( answer )
  else
    writeln( msgNoSol );
end;

procedure Initialize;
begin

  assign( input, inFile );
  reset( input );

  assign( output, outFile );
  rewrite( output );

  {$ifdef Debug}
  assign( dbg, dbgFile );
  rewrite( dbg );
  writeln( '----Initialized----' );
  {$endif}

  new( q );
  new( map );

end;

procedure Finalize;
begin

  {$ifdef Debug}
  close( dbg );
  writeln( '----Finalized----' );
  {$endif}

  close( input );
  close( output );
  halt( 0 );

end;

begin
  Initialize;
  ReadData;
  Solve;
  WriteData;
  Finalize;
end.