/**************************************************************
 * (C) Copyright 2001
 *   Computer Graphics Support Group of 30 Phys-Math Gymnasium
 **************************************************************/

/* FILE NAME   : 13_B.C
 * PROGRAMMER  : Sergey Ivanov (11-2).
 * LAST UPDATE : 04.02.2001
 *
 * No part of this file may be changed without agreement of
 * Computer Graphics Support Group of 30 Phys-Math Gymnasium.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Common types definition */
typedef void VOID;
typedef int BOOL;
typedef char CHAR;
typedef int INT;
typedef short SHORT;
typedef unsigned short USHORT;
typedef long LONG;
typedef unsigned long ULONG;
typedef float FLOAT;

#define TRUE  1
#define FALSE 0

/* Input-output files */
FILE *In, *Out;

/* Point type */
typedef struct tagPOINT
{
  FLOAT X, Y;
} POINT;

/* Polosa type */
typedef struct tagPOL
{
  POINT A, B;
  POINT E, F;
  FLOAT SA, SB;
} POL;

/* Polygon points */
POINT *P;
/* Points count */
INT N = 0;
INT PN = 0;

/* Polosy */
POL *PL;

/* Result count */
INT ResC = 0;
/* Result point */
POINT ResP;

/* Point to line macro */
#define P2L(P, A, B) \
  (((P)->X - (A)->X) * ((B)->Y - (A)->Y) - \
   ((P)->Y - (A)->Y) * ((B)->X - (A)->X))

/* Lines intersection function */
static INT L2L( POINT *LA1, POINT *LB1, POINT *LA2, POINT *LB2, POINT *Res )
{
  FLOAT a1, b1, c1, a2, b2, c2, d;

  a1 = LB1->Y - LA1->Y; /* dy */
  b1 = LA1->X - LB1->X; /* -dx */
  c1 = -LA1->Y * b1 - LA1->X * a1; /* ya*dx - xa*dy */
  a2 = LB2->Y - LA2->Y; /* dy */
  b2 = LA2->X - LB2->X; /* -dx */
  c2 = -LA2->Y * b2 - LA2->X * a2; /* ya*dx - xa*dy */

  d = a1 * b2 - a2 * b1;
  if (d == 0.0)
    return -1;
  Res->Y = (a2 * c1 - a1 * c2) / d;
  Res->X = (b1 * c2 - c1 * b2) / d;
  if (fabs(Res->X) < 0.0001)
    Res->X = 0;
  if (fabs(Res->Y) < 0.0001)
    Res->Y = 0;
  return 0;
} /* End of 'L2L' function */

/* Process function */
static VOID Process( VOID )
{
  INT i, j, k, cnt, max;
  INT c[4];
  POINT A, B, E, F;

  if ((PL = malloc(2 * N * sizeof(PL))) == NULL)
    return;

  /* Find polosy */
  for (i = 0; i < N - 1; i++)
  {
    A = P[i];
    B = P[i + 1];
    if (B.Y == A.Y)
    {
      E.X = A.X;
      E.Y = 1;
      F.X = B.X;
      F.Y = 1;
    }
    else
    {
      E.X = 1;
      E.Y = (A.Y * (B.Y - A.Y) - (E.X - A.X) * (B.X - A.X)) / (B.Y - A.Y);
      F.X = 1;
      F.Y = (B.Y * (B.Y - A.Y) - (F.X - B.X) * (B.X - A.X)) / (B.Y - A.Y);
    }
    PL[i].A = A;
    PL[i].B = B;
    PL[i].E = E;
    PL[i].F = F;
    PL[i].SA = P2L(&A, &B, &F);
    PL[i].SB = P2L(&B, &A, &E);
  }
  /* Find last polosa */
  A = P[i];
  B = P[0];
  if (B.Y == A.Y)
  {
    E.X = A.X;
    E.Y = 1;
    F.X = B.X;
    F.Y = 1;
  }
  else
  {
    E.X = 1;
    E.Y = (A.Y * (B.Y - A.Y) - (E.X - A.X) * (B.X - A.X)) / (B.Y - A.Y);
    F.X = 1;
    F.Y = (B.Y * (B.Y - A.Y) - (F.X - B.X) * (B.X - A.X)) / (B.Y - A.Y);
  }
  PL[i].A = A;
  PL[i].B = B;
  PL[i].E = E;
  PL[i].F = F;
  PL[i].SA = P2L(&A, &B, &F);
  PL[i].SB = P2L(&B, &A, &E);

  /* Find best point */
  ResC = 0;
  for (i = 0; i < N - 1; i++)
  {
    for (j = i + 1; j < N; j++)
    {
      /* Find polosy intersection */
      c[0] = L2L(&PL[i].A, &PL[i].E, &PL[j].A, &PL[j].E, &A);
      c[1] = L2L(&PL[i].A, &PL[i].E, &PL[j].B, &PL[j].F, &B);
      c[2] = L2L(&PL[i].B, &PL[i].F, &PL[j].A, &PL[j].E, &E);
      c[3] = L2L(&PL[i].B, &PL[i].F, &PL[j].B, &PL[j].F, &F);
      cnt = 4 + c[0] + c[1] + c[2] + c[3];
      /* Test all polosy */
      for (k = 0; k < N && cnt > 0; k++)
      {
        /* Test points to poligon-in position */
        if (P2L(&A, &PL[k].B, &PL[k].A) < 0 && c[0] != -1)
          cnt--, c[0] = -1;
        if (P2L(&B, &PL[k].B, &PL[k].A) < 0 && c[1] != -1)
          cnt--, c[1] = -1;
        if (P2L(&E, &PL[k].B, &PL[k].A) < 0 && c[2] != -1)
          cnt--, c[2] = -1;
        if (P2L(&F, &PL[k].B, &PL[k].A) < 0 && c[3] != -1)
          cnt--, c[3] = -1;
        if (cnt == 0)
          continue;
        /* Test points to polosa-in position */
        if (P2L(&A, &PL[k].B, &PL[k].F) * PL[k].SA >= 0 &&
            P2L(&A, &PL[k].A, &PL[k].E) * PL[k].SB >= 0 &&
            c[0] != -1)
          c[0]++;
        if (P2L(&B, &PL[k].B, &PL[k].F) * PL[k].SA >= 0 &&
            P2L(&B, &PL[k].A, &PL[k].E) * PL[k].SB >= 0  &&
            c[1] != -1)
          c[1]++;
        if (P2L(&E, &PL[k].B, &PL[k].F) * PL[k].SA >= 0 &&
            P2L(&E, &PL[k].A, &PL[k].E) * PL[k].SB >= 0 &&
            c[2] != -1)
          c[2]++;
        if (P2L(&F, &PL[k].B, &PL[k].F) * PL[k].SA >= 0 &&
            P2L(&F, &PL[k].A, &PL[k].E) * PL[k].SB >= 0 &&
            c[3] != -1)
          c[3]++;
      }
      if (cnt == 0)
        continue;
      max = 0;
      for (k = 0; k < 4; k++)
        if (c[k] > c[max])
          max = k;
      if (c[max] > ResC)
      {
        ResC = c[max];
        switch (max)
        {
        case 0:
          ResP = A;
          break;
        case 1:
          ResP = B;
          break;
        case 2:
          ResP = E;
          break;
        case 3:
          ResP = F;
          break;
        }
      }
    }
  }
  /* Clear memory */
  free(PL);
} /* End of 'Process' function */

/* The main function */
INT main( VOID )
{
  INT i;
  FLOAT x, y;

  In = fopen("input.txt", "rt");
  Out = fopen("output.txt", "wt");

  fscanf(In, "%d", &N);
  if ((P = malloc(N * sizeof(POINT))) == NULL)
  {
    fclose(In);
    fclose(Out);
    return -1;
  }
  for (i = 0; i < N; i++)
  {
    fscanf(In, "%f%f", &x, &y);
    P[i].X = x;
    P[i].Y = y;
  }

  Process();

  fprintf(Out, "%d\n", ResC);
  fprintf(Out, "%f %f", ResP.X, ResP.Y);

  free(P);
  fclose(In);
  fclose(Out);
  return 0;
} /* End of 'main' function */

/* END OF '13_B.C' FILE */
