#include <stdio.h>
#include <math.h>
#include <stdlib.h>

const float Eps = 0.0001;

struct Line {
  float A,B,C;
  Line() { A = 0; B = 0; C = 0; };
  Line(float X1,float Y1, float X2, float Y2) {
    Init(X1,Y1,X2,Y2);
  }
  void Init(float X1,float Y1, float X2, float Y2) {
    A = Y2 - Y1;
    B = X1 - X2;
    float L = sqrt(A*A + B*B);
    A /= L; B /= L;
    C = -(A*X1 + B*Y1);
  };
  Line MakePerp(float X, float Y, int Right) {
    if (Right) return Line(X,Y,X+A,Y+B);
    else return Line(X,Y,X-A,Y-B);
  };
  float SDist(float X, float Y) {
    return A*X + B*Y + C;
  };
};

struct Otrezok {
  float X1, Y1;
  float X2, Y2;
  Line L;
  Otrezok() {
    X1 = Y1 = X2 = Y2 = 0;
  };
  void Init(float iX1, float iY1, float iX2, float iY2) {
    X1 = iX1; Y1 = iY1; X2 = iX2; Y2 = iY2;
    L.Init(X1,Y1,X2,Y2);
  };
};

struct Bounds {
  Line BoundS, BoundE;
  Otrezok Otr;

  void CalcBounds() {
    BoundS = Otr.L.MakePerp(Otr.X1, Otr.Y1, 0);
    BoundE = Otr.L.MakePerp(Otr.X2, Otr.Y2, 1);
  };
  int InBound(float X, float Y) {
    return (BoundS.SDist(X,Y) > -Eps) &&
           (BoundE.SDist(X,Y) > -Eps);
  };
};


Bounds Sides[100];
int NumSides = 0;

void LoadLines() {
  float XF, YF,XP, YP, XC, YC;
  FILE *In = fopen("input.txt","rt");
  fscanf(In, "%d", &NumSides);
  if (NumSides > 100) {
    printf("FATAL: NumSides > 100\n");
    exit(1);
  };
  fscanf(In, "%f %f",&XF, &YF);
  XP = XF; YP = YF;
  for(int i = 0; i < NumSides-1; i++) {
    fscanf(In, "%f %f",&XC, &YC);
    Sides[i].Otr.Init(XP,YP,XC,YC);
    XP = XC; YP = YC;
  };
  Sides[NumSides-1].Otr.Init(XP,YP,XF,YF);
  fclose(In);
};

int CrossLines(Line L1, Line L2, float *X, float *Y) {
  float D = L1.A*L2.B - L2.A*L1.B;
  if (fabs(D) < Eps) return 0;
  *X = (L1.B*L2.C - L1.C*L2.B)/D;
  *Y = (L2.A*L1.C - L1.A*L2.C)/D;
  return 1;
};

float RX=0,RY=0;
int N=0;

void ProcessPnt(float X, float Y, int TF = 0, int TS = 1) {
  if (Sides[TF].Otr.L.SDist(X,Y) > Eps) return; // out of poly
  if (Sides[TS].Otr.L.SDist(X,Y) > Eps) return; // out of poly
  for(int i = 0; i < NumSides; i++)
     if (Sides[i].Otr.L.SDist(X,Y) > Eps) return; // out of poly
  int Num = 0;
  for(i = 0; i < NumSides; i++)
   if (Sides[i].InBound(X,Y)) Num++;
  if (Num > N) {
    N = Num;
    RX = X; RY = Y;
  };
};

void DoProcess() {
  for(int i = 0; i < NumSides-1; i++)
   for(int j = i+1; j < NumSides; j++) {
     float X,Y;
     if (CrossLines(Sides[i].BoundS,Sides[j].BoundS,&X,&Y)) {
       ProcessPnt(X,Y,i,j);
       if (CrossLines(Sides[i].BoundE,Sides[j].BoundS,&X,&Y)) ProcessPnt(X,Y,i,j);
       if (CrossLines(Sides[i].BoundS,Sides[j].BoundE,&X,&Y)) ProcessPnt(X,Y,i,j);
       if (CrossLines(Sides[i].BoundE,Sides[j].BoundE,&X,&Y)) ProcessPnt(X,Y,i,j);
     };
   };
};

int main() {
  LoadLines();
  for(int i = 0; i < NumSides; i++) Sides[i].CalcBounds();
  DoProcess();
  FILE *Out = fopen("output.txt","wt");
  fprintf(Out,"%d\n%f %f\n",N,RX,RY);
  fclose(Out);
  return 0;
};