#define ERROR								-1
#define RUN								   0
#define COMPILE						   1
#define DEBUG								2
#define FUNCTION                    3
#define ENDFUNCTION                 4
#define RETURN                      5
#define PRINT                       6
#define IF                          7
#define MOVE                        8
#define BRACKET                     9
#define EQUATE                      10
#define LESS                        11
#define MORE                        12
#define NUMBER                      13
#define CONST	                     14
#define END	                        15
#define PLUS	                     16
#define MINUS	                     17
#define MUL	                        18
#define SINGLE	                     19
#define LETTER	                     20
#define THEN	                     21

#include <stdio.h>
#include <string.h>

struct expr
 {
   int operat;
   int flag [2];
   int num [2];
   char param [30];
   char name, son [2];
   int state, answer;
 };


struct func
 {
   struct expr command [256], answer;
   char param [256];
   int commands, params;
 } function [128];


int init ();
void deinit ();
void compile ();
void translate ();
void debug ();
int run (struct func * func);
int get_lexem ();
struct func * get_func ();
void print_func (struct func * func);
void get_expr (struct expr * expr);
void print_expr (struct expr * expr);
int count_expr (struct expr * expr);
char chrlwr (char a) { return ((a <= 'Z') ? (a - 'A' + 'a') : a); }
void get_nothing (struct expr * expr, int side);
void put_nothing (struct expr * expr, int side);
int count_nothing (struct expr * expr, int side);


FILE * input, * output;
int functions [256];
int var [256] [256], param [256];
int last_number, stack, last_flag, last_lexem;
char last_letter, letter [256];


main ()
 {
   switch (init ())
    {
    	case RUN:
       compile ();
       run (& (function [0]));
       break;
    	case COMPILE:
       compile ();
       translate ();
       break;
    	case DEBUG:
       compile ();
       debug ();
       break;
    }
    
   deinit ();
   return (0);
 }



int init ()
 {
   char str [100];
 	input = fopen ("INPUT.TXT", "r");
 	output = fopen ("OUTPUT.TXT", "w");
   fscanf (input, "%s", str);
   strupr (str); stack = 0;
   function [0]. params = last_flag = 0;
   memset (functions, 0, 1024);
   memset (letter, 0, 256);

   for (int i = 'a'; i <= 'z'; i ++)
    letter [i] = letter [i - 'a' + 'A'] = 1;

   if (! strcmp (str, "RUN")) return (RUN);
   if (! strcmp (str, "COMPILE")) return (COMPILE);
   if (! strcmp (str, "DEBUG")) return (DEBUG);

   return (ERROR);
 }


void deinit ()
 {
   fclose (input);
   fclose (output);
 }


void compile ()
 {
   int lexem;
   char save;
   struct func * cur_func = & function [0];
   cur_func -> commands = 0;
   
   while ((lexem = get_lexem ()) != ERROR)
    {
      switch (lexem)
       {
         case FUNCTION:
          cur_func = get_func ();
          break;
         case ENDFUNCTION:
          cur_func = & function [0];
          break;
         case RETURN:
          cur_func -> command [cur_func -> commands ++]. state = RETURN;
          get_expr (& cur_func -> answer);
          break;
         case PRINT:
          get_expr (& (cur_func -> command [cur_func -> commands]));
          cur_func -> command [cur_func -> commands ++]. state = PRINT;
          break;
         case IF:
          get_expr (& (cur_func -> command [cur_func -> commands]));
          cur_func -> command [cur_func -> commands ++]. state = IF;
          break;
         case END: break;
         case THEN: break;
         case ERROR: break;
         default:
          save = chrlwr (last_letter);
          get_lexem ();
          get_expr (& (cur_func -> command [cur_func -> commands]));
			 cur_func -> command [cur_func -> commands]. state = MOVE;
			 cur_func -> command [cur_func -> commands ++]. name = save;
          break;
       }
    }
 }


void translate ()
 {
   int counter, j;
   
 	fprintf (output, "#include <stdio.h>\n\nFILE * file;\n\n");

   for (int i = 1; i < 256; i ++)
    if (functions [i])
     {
      fprintf (output, "int %c (", i);
      counter = function [i]. params - 1;
      
      for (j = 0; counter && (j < 256); j ++)
       if (function [i]. param [j])
        fprintf (output, "%c, ", j), counter --;

      for (; j < 256; j ++)
       if (function [i]. param [j])
        fprintf (output, "%c", j);

      fprintf (output, ")  {\n");
      print_func (& function [i]);
      fprintf (output, "\n }\n\n");
     }

   fprintf (output, "main ()\n {\n   file = fopen (\"OUTPUT.TXT\", \"w\");\n");
   print_func (& function [0]);
   fprintf (output, "   fclose (file);\n   return (0);\n }");
 }


void print_func (struct func * func)
 {
 	for (int i = 0; i < func -> commands; i ++)
    {
      switch (func -> command [i]. state)
       {
       	case PRINT:
          fprintf (output, "   fprintf (file, \"%%d\", ");
          print_expr (& (func -> command [i]));
          fprintf (output, ");\n");
          break;
       	case RETURN:
          fprintf (output, "   return (");
          print_expr (& (func -> answer));
          fprintf (output, ");\n");
          break;
       	case IF:
          fprintf (output, "   if (");
          print_expr (& (func -> command [i]));
          fprintf (output, ")");
          break;
         case MOVE:
          fprintf (output, "   %c = ", func -> command [i]. name);
          print_expr (& (func -> command [i]));
          fprintf (output, ";\n");
       }
    }
 }


int run (struct func * func)
 {
   memset (var [stack], 0, 1024);

   for (int i = 0; i < 256; i ++)
    if (func -> param [i])
     var [stack] [i] = param [i];

   for (int i = 0; i < func -> commands; i ++)
    switch (func -> command [i]. state)
     {
     	 case PRINT:
        fprintf (output, "%d\n", count_expr (& (func -> command [i])));
        break;
     	 case RETURN:
        return (count_expr (& (func -> command [i])));
     	 case IF:
        if (count_expr (& (func -> command [i])) == 0)
         do
          i ++;
         while (func -> command [i]. state == IF);
         
        break;
       case MOVE:
        var [stack] [func -> command [i]. name] = count_expr (& (func -> command [i]));
     }
 }


struct func * get_func ()
 {
   get_lexem ();
   struct func * func = & function [chrlwr (last_letter)];
   functions [chrlwr (last_letter)] = 1;
   func -> commands = 0;
   get_lexem ();
   
   for (func -> params = 0; 1; func -> params ++)
    {
      if (get_lexem () == BRACKET)
       {
         get_lexem ();
         return (func);
       }
      else func -> param [chrlwr (last_letter)] = 1;
    }
 }


int get_lexem ()
 {
   if (last_flag)
    {
      last_flag = 0;
      return (last_lexem);
    }

   last_flag = 0;
   int flag = 0;
   char str [1000], chr;
   
   do
    {
      if (flag)
       while ((fgetc (input) != '\n') && (! feof (input)));
       
      flag = 1;
      
	   do
   	 last_letter = fgetc (input);
	   while ((last_letter == ' ') || (last_letter == '\t') || (last_letter == ','));
    }
   while (last_letter == '#');

   if (last_letter == -1)
    return (ERROR);

   switch (last_letter)
    {
      case '+': return (last_lexem = PLUS);
      case '-': return (last_lexem = MINUS);
      case '*': return (last_lexem = MUL);
      case '=': return (last_lexem = EQUATE);
      case '<': return (last_lexem = LESS);
      case '>': return (last_lexem = MORE);
      case ')': return (last_lexem = BRACKET);
      case '(': return (last_lexem = BRACKET);
      case '}': return (last_lexem = ENDFUNCTION);
      case '\n': return (last_lexem = END);
    }
    
   fseek (input, -1, SEEK_CUR);

   if ((last_letter >= '0') && (last_letter <= '9'))
    {
      fscanf (input, "%d", & last_number);
      return (last_lexem = NUMBER);
    }

   str [0] = 0;

   for (int i = 0; letter [chr = fgetc (input)]; i ++)
    str [i] = chr, str [i + 1] = 0;

   if (str [0])
    fseek (input, -1, SEEK_CUR);
    
   strupr (str);

   if (! strcmp (str, "FUNCTION")) return (last_lexem = FUNCTION);
   if (! strcmp (str, "RETURN")) return (last_lexem = RETURN);
   if (! strcmp (str, "PRINT")) return (last_lexem = PRINT);
   if (! strcmp (str, "IF")) return (last_lexem = IF);
   if (! strcmp (str, "THEN")) return (last_lexem = THEN);

   if (str [0])
    last_letter = chrlwr (str [0]);

   if (feof (input)) return (last_lexem = ERROR);
   else return (last_lexem = LETTER);
 }


void get_expr (struct expr * expr)
 {
   int save;
   get_nothing (expr, 0);

   if (((save = get_lexem ()) == END) || (save == THEN))
    {
      expr -> operat = SINGLE;
      return;
    }

   expr -> operat = save;
   get_nothing (expr, 1);
 }


void get_nothing (struct expr * expr, int side)
 {
   int lexem = get_lexem ();
   char save;

   if (lexem == MINUS)
    {
      get_lexem ();
      expr -> num [side] = -last_number;
      expr -> flag [side] = NUMBER;
    }
   else if ((lexem == PLUS) || (lexem == NUMBER))
    {
      get_lexem ();
      expr -> num [side] = last_number;
      expr -> flag [side] = NUMBER;
    }
   else
    {
      save = last_letter;
      
      if (get_lexem () == BRACKET)
       {
        expr -> flag [side] = FUNCTION;
	     expr -> son [side] = chrlwr (last_letter);
   	  struct func * func = & function [chrlwr (last_letter)];

	     for (int i = 0; i < func -> params; i ++)
   	   {
      	  get_lexem ();
           expr -> param [i] = chrlwr (last_letter);
	      }

        get_lexem ();
       }
      else
       {
         last_flag = 1;
         expr -> flag [side] = CONST;
         expr -> son [side] = chrlwr (save);
       }
    }
 }

 
void print_expr (struct expr * expr)
 {
   put_nothing (expr, 0);

   if (expr -> operat == SINGLE)
     return;

   switch (expr -> operat)
    {
      case PLUS:
         fprintf (output, " + ");
         break;
      case MINUS:
         fprintf (output, " - ");
         break;
      case MUL:
         fprintf (output, " * ");
         break;
      case EQUATE:
         fprintf (output, " == ");
         break;
      case LESS:
         fprintf (output, " < ");
         break;
      case MORE:
         fprintf (output, " > ");
    }
    
   put_nothing (expr, 1);
 }


void put_nothing (struct expr * expr, int side)
 {
   switch (expr -> flag [side])
    {
      case NUMBER:
       fprintf (output, "%d", expr -> num [side]);
       break;
      case FUNCTION:
       fprintf (output, "%c (", expr -> son [side]);

       for (int i = 0; i < function [expr -> son [side]]. params - 1; i ++)
        fprintf (output, "%c, ", expr -> param [i]);

       if (function [expr -> son [side]]. params)
        fprintf (output, "%c", expr -> param [function [expr -> son [side]]. params - 1]);

       fprintf (output, ")");
       break;
      case CONST:
       fprintf (output, "%c", expr -> son [side]);
    }
 }

 
int count_expr (struct expr * expr)
 {
   count_nothing (expr, 0);

   if (expr -> operat = SINGLE)
    return (expr -> num [0]);

   count_nothing (expr, 1);

   switch (expr -> operat)
    {
      case PLUS:   return (expr -> num [0] + expr -> num [1]);
      case MINUS:  return (expr -> num [0] - expr -> num [1]);
      case MUL:    return (expr -> num [0] * expr -> num [1]);
      case EQUATE: return (expr -> num [0] == expr -> num [1]);
      case LESS:   return (expr -> num [0] < expr -> num [1]);
      case MORE:   return (expr -> num [0] > expr -> num [1]);
    }
 }


int count_nothing (struct expr * expr, int side)
 {
   switch (expr -> flag [side])
    {
      case NUMBER: return (expr -> num [side]);
      case FUNCTION:
       {
       struct func * func = & (function [expr -> son [side]]);
       
       for (int i = 0; i < func -> params; i ++)
        param [i] = var [stack] [func -> param [i]];
      
       stack ++;
       run (func);
       stack --;
       break;
      }
      case CONST:
       return (var [stack] [expr -> son [side]]);
    }
 }


void debug ()
 {
 }

 

