#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>

#define MAXL 210

struct
{
  enum
  {
    Print,
    If,
    Then,
    Function,
    Return,
    Variable,
    Constant,
    Plus,
    Minus,
    Multiply,
    Less,
    Equal,
    Greater,
    Left_par,
    Right_par,
    Left_brace,
    Right_brace,
    Comma
  } type;
  char var;
  int value;
} t;

int eol_passed;
int get( void )
{
  int c;

  eol_passed = 0;
  while (isspace(c = getchar()))
    if (c == '\n')
    {
      eol_passed = 1;
      c = getchar();
      if (c == '#')
        while ((c = getchar()) != '\n')
          if (c == EOF)
            return EOF;
      if (c == EOF)
        return EOF;
      ungetc(c, stdin);
    }
  if (c == EOF)
    return EOF;
  return toupper(c);
}

void skip_c( int i )
{
  while (i-- > 0)
    getchar();
}

int skip, scan_minus;
int get_token( void )
{
  if (skip)
    skip = 0;
  else
  {
    int c = get();
  
    switch (c)
    {
    case EOF:
      return 0;
    case '+':
      t.type = Plus;
      break;
    case '*':
      t.type = Multiply;
      break;
    case '<':
      t.type = Less;
      break;
    case '=':
      t.type = Equal;
      break;
    case '>':
      t.type = Greater;
      break;
    case '(':
      t.type = Left_par;
      break;
    case ')':
      t.type = Right_par;
      break;
    case '{':
      t.type = Left_brace;
      break;
    case '}':
      t.type = Right_brace;
      break;
    case ',':
      t.type = Comma;
      break;
    default:
      if (isalpha(c))
      {
        int ch = getchar();
  
        if (isalpha(ch))
          switch (c)
          {
          case 'P':
            t.type = Print;
            skip_c(3);
            break;
          case 'I':
            t.type = If;
            break;
          case 'T':
            t.type = Then;
            skip_c(2);
            break;
          case 'F':
            t.type = Function;
            skip_c(6);
            break;
          case 'R':
            t.type = Return;
            skip_c(4);
            break;
          }
        else
        {
          ungetc(ch, stdin);
          t.type = Variable;
          t.var = c;
        }
      }
      else if (c == '-' && scan_minus)
        t.type = Minus;
      else
      {
        ungetc(c, stdin);
        scanf("%i", &t.value);
      }
      break;
    }
  }
  scan_minus = 0;
  return 1;
}

typedef struct
{
  enum
  {
    Var, Const
  } type;
  char var;
  int value;
} Smth;

typedef struct Stmt
{
  enum
  {
    Assign, Prn, Cond, Ret
  } type;
  struct
  {
    char var;
    enum
    {
      Cons, Oper, Fcall
    } type;
    Smth operand;
    Smth left_op, right_op;
    int op;
    char fname;
    int par_no;
    Smth *par;
  } assign_op;
  Smth print_op;
  struct
  {
    char var;
    struct Stmt *command;
  } cond_op;
  Smth ret_op;
} Stmt;

typedef struct
{
  char var[26];
  int var_no;
  Stmt stmts[MAXL];
  int stmts_no;
} Func;

Func funcs[256];
Stmt stmts[MAXL];
int funcs_no, stmts_no;
char func_seq[26];

Smth get_smth( void )
{
  Smth r;

  get_token();
  if (t.type == Variable)
  {
    r.type = Var;
    r.var = t.var;
  }
  else
  {
    r.type = Const;
    r.value = t.value;
  }
  return r;
}

Stmt get_statement( void )
{
  Stmt r;
  
  get_token();
  switch (t.type)
  {
  case Variable:
    {
      Smth op;
      
      r.type = Assign;
      r.assign_op.var = t.var;
      get_token();
      op = get_smth();
      scan_minus = 1;
      get_token();
      if (eol_passed)
      {
        skip = 1;
        r.assign_op.type = Cons;
        r.assign_op.operand = op;
      }
      else if (t.type == Left_par)
      {
        Smth par[26];
        int par_no = 0;
        
        r.assign_op.type = Fcall;
        r.assign_op.fname = op.var;
        while (1)
        {
          get_token();
          if (t.type == Right_par)
            break;
          skip = 1;
          par[par_no++] = get_smth();
          get_token();
          if (t.type == Right_par)
            break;
        }
        r.assign_op.par_no = par_no;
        r.assign_op.par = malloc(par_no * sizeof(Smth));
        memcpy(r.assign_op.par, par, par_no * sizeof(Smth));
      }
      else
      {
        r.assign_op.type = Oper;
        r.assign_op.op = t.type;
        r.assign_op.left_op = op;
        r.assign_op.right_op = get_smth();
      }
      break;
    }
  case Print:
    r.type = Prn;
    r.print_op = get_smth();
    break;
  case Return:
    r.type = Ret;
    r.ret_op = get_smth();
    break;
  case If:
    r.type = Cond;
    get_token();
    r.cond_op.var = t.var;
    get_token();
    r.cond_op.command = malloc(sizeof(Stmt));
    *(Stmt *)r.cond_op.command = get_statement();
    break;
  }
  return r;
}

void get_function( Func *r )
{
  r->var_no = 0;
  r->stmts_no = 0;
  get_token();
  while (1)
  {
    get_token();
    if (t.type == Right_par)
      break;
    r->var[r->var_no++] = t.var;
    get_token();
    if (t.type == Right_par)
      break;
  }
  get_token();
  while (1)
  {
    get_token();
    if (t.type == Right_brace)
      break;
    skip = 1;
    r->stmts[r->stmts_no++] = get_statement();
  }
}

void parse( void )
{
  while (1)
  {
    if (!get_token())
      break;
    switch (t.type)
    {
    case Function:
      {
        char c;
        
        get_token();
        get_function(&funcs[c = t.var]);
        func_seq[funcs_no++] = c;
        break;
      }
    case Variable:
    case Print:
    case If:
    case Return:
      skip = 1;
      stmts[stmts_no++] = get_statement();
      break;
    }
  }
}

void print_smth( Smth s )
{
  if (s.type == Var)
    printf("%c", s.var);
  else
    printf("%i", s.value);
}

void compile_statement( Stmt s )
{
  switch (s.type)
  {
  case Assign:
    printf("%c = ", s.assign_op.var);
    switch (s.assign_op.type)
    {
    case Cons:
      print_smth(s.assign_op.operand);
      printf(";\n");
      break;
    case Oper:
      print_smth(s.assign_op.left_op);
      switch (s.assign_op.op)
      {
      case Plus:
        printf(" + ");
        break;
      case Minus:
        printf(" - ");
        break;
      case Multiply:
        printf(" * ");
        break;
      case Less:
        printf(" < ");
        break;
      case Equal:
        printf(" == ");
        break;
      case Greater:
        printf(" > ");
        break;
      }
      print_smth(s.assign_op.right_op);
      printf(";\n");
      break;
    case Fcall:
      {
        int j;
        
        printf("f%c(", s.assign_op.fname);
        if (s.assign_op.par_no != 0)
        {
          print_smth(s.assign_op.par[0]);
          for (j = 1; j < s.assign_op.par_no; j++)
            printf(", "), print_smth(s.assign_op.par[j]);
        }
        printf(");\n");
      }
      break;
    }
    break;
  case Prn:
    printf("printf(\"%%i\\n\", ");
    print_smth(s.print_op);
    printf(");\n");
    break;
  case Cond:
    printf("if (%c)\n", s.cond_op.var);
    compile_statement(*s.cond_op.command);
    break;
  case Ret:
    printf("return ");
    print_smth(s.ret_op);
    printf(";\n");
    break;
  }
}

void compile( void )
{
  int i, j, print_c = 0;
  
  printf("#include <stdio.h>\n\n");
  for (i = 0; i < funcs_no; i++)
  {
    int fc = func_seq[i];
    Func f = funcs[fc];
    int print_c = 0;

    printf("int f%c( ", fc);
    if (f.var_no == 0)
      printf("void");
    else
    {
      for (j = 0; j < f.var_no - 1; j++)
        printf("int %c, ", f.var[j]);
      printf("int %c", f.var[f.var_no - 1]);
    }
    printf(" )\n{\n");
    for (j = 'A'; j <= 'Z'; j++)
      if (!strchr(f.var, j))
        if (print_c)
          printf(", %c = 0", j);
        else
          print_c = 1, printf("int %c = 0", j);
    printf(";\n\n");
    for (j = 0; j < f.stmts_no; j++)
      compile_statement(f.stmts[j]);
    printf("}\n\n");
  }
  printf("int main( void )\n"
         "{\n"
         "int A = 0, B = 0, C = 0, D = 0, E = 0, F = 0, G = 0, H = 0, I = 0, J = 0, K = 0, L = 0, M = 0, N = 0, O = 0, P = 0, Q = 0, R = 0, S = 0, T = 0, U = 0, V = 0, W = 0, X = 0, Y = 0, Z = 0;\n"
         "\n"
         "freopen(\"OUTPUT.TXT\", \"w\", stdout);\n");
  for (j = 0; j < stmts_no; j++)
    compile_statement(stmts[j]);
  printf("return 0;\n}\n\n");
}

int stack[250][256], pos;

void set_var_val( char v, int val )
{
  stack[pos][v] = val;
}

int get_var_val( char v )
{
  return stack[pos][v];
}

int get_smth_val( Smth s )
{
  if (s.type == Const)
    return s.value;
  return get_var_val(s.var);
}

int run_statement( Stmt s, int *result );

int call_function( char f, Smth *par )
{
  int i, res;
  int var_no = funcs[f].var_no;
  char *var = funcs[f].var;
  int stmts_no = funcs[f].stmts_no;
  Stmt *stmts = funcs[f].stmts;

  memset(stack[pos + 1], 0, sizeof(stack[pos + 1]));
  for (i = 0; i < var_no; i++)
    stack[pos + 1][var[i]] = get_smth_val(par[i]);
  pos++;
  for (i = 0; i < stmts_no; i++)
    if (run_statement(stmts[i], &res))
    {
      pos--;
      return res;
    }
}

int run_statement( Stmt s, int *result )
{
  switch (s.type)
  {
  case Assign:
    switch (s.assign_op.type)
    {
    case Cons:
      set_var_val(s.assign_op.var, get_smth_val(s.assign_op.operand));
      break;
    case Oper:
      switch (s.assign_op.op)
      {
      case Plus:
        set_var_val(s.assign_op.var, get_smth_val(s.assign_op.left_op)
                                     + get_smth_val(s.assign_op.right_op));
        break;
      case Minus:
        set_var_val(s.assign_op.var, get_smth_val(s.assign_op.left_op)
                                     - get_smth_val(s.assign_op.right_op));
        break;
      case Multiply:
        set_var_val(s.assign_op.var, get_smth_val(s.assign_op.left_op)
                                     * get_smth_val(s.assign_op.right_op));
        break;
      case Less:
        set_var_val(s.assign_op.var, get_smth_val(s.assign_op.left_op)
                                     < get_smth_val(s.assign_op.right_op));
        break;
      case Equal:
        set_var_val(s.assign_op.var, get_smth_val(s.assign_op.left_op)
                                     == get_smth_val(s.assign_op.right_op));
        break;
      case Greater:
        set_var_val(s.assign_op.var, get_smth_val(s.assign_op.left_op)
                                     > get_smth_val(s.assign_op.right_op));
        break;
      }
      break;
    case Fcall:
      set_var_val(s.assign_op.var, call_function(s.assign_op.fname,
                                                 s.assign_op.par));
      break;
    }
    break;
  case Prn:
    printf("%i\n", get_smth_val(s.print_op));
    break;
  case Cond:
    if (get_var_val(s.cond_op.var))
      if (run_statement(*s.cond_op.command, result))
        return 1;
    break;
  case Ret:
    *result = get_smth_val(s.ret_op);
    return 1;
  }
  return 0;
}

void run( void )
{
  int j;
  
  for (j = 0; j < stmts_no; j++)
    run_statement(stmts[j], 0);
}

void debug( void )
{
}

int main( void )
{
  int c, ch;
  
  freopen("INPUT.TXT", "r", stdin);
  c = get();
  while ((ch = getchar()) != '\n')
    ;
  ungetc('\n', stdin);
  parse();
  if (c == 'R')
  {
    freopen("OUTPUT.TXT", "w", stdout);
    run();
  }
  if (c == 'C')
  {
    freopen("OUTPUT.c", "w", stdout);
    compile();
  }
  if (c == 'D')
    debug();
  return 0;
}

