{$I-,Q-,R-,S-,V-,X-}
Program FreeVMAG;
Uses SysUtils,Crt;
Const
  InFile='INPUT.TXT';
  OutFile='OUTPUT.TXT';
  OutProg='OUTPUT.PAS';
  HelpText:Array[0..18]Of String=(
    'Welcome to the Free-VMAG debugger!',
    '',
    'Keys:',
    '  F1 - this help',
    '  F7 - trace into all function calls',
    '  F8 - step over all function calls',
    '  F9 - run program until finish',
    '  F10 - run program until return of current function',
    '  Left arrow - scroll program text to the left',
    '  Right arrow - scroll program text to the right',
    '  Ctrl+F2 - interrupt program execution and exit debugger',
    '',
    'Colors of watches (right part of the screen):',
    '  red - variables that were modified since press of F7, F8, F9 or F10',
    '  green - variables that are used in current line',
    '  yellow - variables that might be red and green at the same time',
    '  white - all other variables',
    '',
    'Press any key to continue ...');
Const
  {Modes&Lexems}
  _Run=1;
  _Compile=2;
  _Debug=3;
  {Lexems}
  _Number=4;
  _Identifier=5;
  _ORoundBracket=6;
  _CRoundBracket=7;
  _OFigureBracket=8;
  _CFigureBracket=9;
  _Print=10;
  _If=11;
  _Then=12;
  _Function=13;
  _Return=14;
  _Operator=15;
  _Comma=16;
  _EOF=17;
  LexName:Array[1..16]Of String=('Run','Compile','Debug','Number','Identifier','(',')','{','}','Print','If','Then','Function','Return','Operator','EOF');
  MaxData=256;
  MaxStack=210;
  MaxWrote=10002;
  MaxString=210;
Type
  TVars=Array['a'..'z']Of LongInt;
  PCommand=^TCommand;
  TCommand=Record
    Kind:Byte;
    Data:Array[0..MaxData-1]Of Char;
    Valu:Array[0..MaxData-1]Of LongInt;
    RefCmd:PCommand;
    LineNumber:LongInt;
    Next:PCommand;
  End;
  PFunction=^TFunction;
  TFunction=Record
    FirstCmd:PCommand;
    Name:Char;
    ParCnt:LongInt;
    ParNam:Array[0..MaxData-1]Of Char;
    Next:PFunction;
  End;
  TStack=Record
    Vars:TVars;
    RetPoint:PCommand;
  End;
Var
  Txt,Prg:Text;
  TxtO,PrgO:Boolean;
  Mode:Byte;
  CurLex:Byte;
  CurCh:Char;
  CurValue:LongInt;
  CurTmp,CurSt:String;
  LineID:LongInt;
  Funcs:PFunction;
  Cmds:PCommand;
  Source:Array[1..MaxString]Of String;
  NoPrint:Boolean;
  NSource:LongInt;
  Stack:Array[0..MaxStack]Of TStack;
  StkPos:LongInt;
  Wrote:Array[0..MaxWrote]Of LongInt;
  NWrote:LongInt;
  Vars:TVars;
Procedure CloseAll;
Begin
  Close(Input);
  If TxtO Then Close(Txt);
  If PrgO Then Close(Prg);
End;
Procedure Error(Const S:String);
Begin
  If Mode=_Debug Then Begin
    TextColor(7);
    TextBackGround(0);
    ClrScr;
  End;
  WriteLn('ERROR! At line ',LineID,': ',S);
  CloseAll;
  Halt;
End;
Procedure FreeAll;
Var
  PPC,PC:PCommand;
  PPF,PF:PFunction;
Begin
  PC:=Cmds;
  While PC<>Nil Do Begin
    PPC:=PC;
    If PC^.RefCmd=Nil Then
      PC:=PC^.Next
    Else
      PC:=PC^.RefCmd;
    Dispose(PPC);
  End;
  PF:=Funcs;
  While PF<>Nil Do Begin
    PPF:=PF;
    PF:=PF^.Next;
    PC:=PPF^.FirstCmd;
    While PC<>Nil Do Begin
      PPC:=PC;
      If PC^.RefCmd=Nil Then
        PC:=PC^.Next
      Else
        PC:=PC^.RefCmd;
      Dispose(PPC);
    End;
    Dispose(PPF);
  End;
End;
Procedure HelpScreen;
Var
  I:LongInt;
Begin
  TextColor(15);
  TextBackGround(1);
  Window(1,2,80,24);
  ClrScr;
  Window(1,1,80,25);
  For I:=Low(HelpText) To High(HelpText) Do Begin
    GotoXY(2,2+I);
    Write(HelpText[I]);
  End;
End;
Function ScanString:Boolean;
Begin
  ScanString:=False;
  While Not EOF Do Begin
    ReadLn(CurSt);
    Inc(LineID);
    Inc(NSource);
    Source[NSource]:=CurSt;
    If (CurSt<>'') And (CurSt[1]='#') Then Continue;
    CurSt:=LowerCase(Trim(CurSt));
    If CurSt='' Then Continue;
    ScanString:=True;
    Break;
  End;
End;
Procedure ScanLexem(ReadingNumber:Boolean);
Var
  I:LongInt;
  Err:Word;
  Sgn:LongInt;
Begin
  If CurSt='' Then
    If Not ScanString Then Begin CurLex:=_EOF; Exit; End;
  Case CurSt[1] Of
    'a'..'z':Begin
      I:=1;
      While (I<=Length(CurSt)) And (CurSt[I]>='a') And (CurSt[I]<='z') Do Inc(I);
      CurTmp:=Copy(CurSt,1,I-1);
      Delete(CurSt,1,I-1);
      If I=2 Then Begin
        CurLex:=_Identifier;
        CurCh:=CurTmp[1];
      End Else Begin
        If CurTmp='print' Then CurLex:=_Print Else
        If CurTmp='if' Then CurLex:=_If Else
        If CurTmp='then' Then CurLex:=_Then Else
        If CurTmp='function' Then CurLex:=_Function Else
        If CurTmp='return' Then CurLex:=_Return Else
        If CurTmp='run' Then CurLex:=_Run Else
        If CurTmp='compile' Then CurLex:=_Compile Else
        If CurTmp='debug' Then CurLex:=_Debug Else
          Error('Unknown word: "'+CurTmp+'"');
      End;
    End;
    '=','<','>','*':Begin
      CurLex:=_Operator;
      CurCh:=CurSt[1];
      Delete(CurSt,1,1);
    End;
    '(':Begin
      CurLex:=_ORoundBracket;
      Delete(CurSt,1,1);
    End;
    ')':Begin
      CurLex:=_CRoundBracket;
      Delete(CurSt,1,1);
    End;
    '{':Begin
      CurLex:=_OFigureBracket;
      Delete(CurSt,1,1);
    End;
    '}':Begin
      CurLex:=_CFigureBracket;
      Delete(CurSt,1,1);
    End;
    ',':Begin
      CurLex:=_Comma;
      Delete(CurSt,1,1);
    End;
    '0'..'9':Begin
      I:=1;
      While (I<=Length(CurSt)) And (CurSt[I]>='0') And (CurSt[I]<='9') Do Inc(I);
      CurTmp:=Copy(CurSt,1,I-1);
      Delete(CurSt,1,I-1);
      Val(CurTmp,CurValue,Err);
      If Err>0 Then
        Error('Invalid number: "'+CurTmp+'"');
      CurLex:=_Number;
    End;
    '-','+':Begin
      If ReadingNumber Then Begin
        If CurSt[1]='-' Then Sgn:=-1 Else Sgn:=1;
        Delete(CurSt,1,1);
        CurSt:=Trim(CurSt);
        I:=1;
        While (I<=Length(CurSt)) And (CurSt[I]>='0') And (CurSt[I]<='9') Do Inc(I);
        CurTmp:=Copy(CurSt,1,I-1);
        Delete(CurSt,1,I-1);
        Val(CurTmp,CurValue,Err);
        If Err>0 Then
          Error('Invalid number: "'+CurTmp+'"');
        CurValue:=CurValue*Sgn;
        CurLex:=_Number;
      End Else Begin
        CurLex:=_Operator;
        CurCh:=CurSt[1];
        Delete(CurSt,1,1);
      End;
    End;
    Else Error('Invalid character: "'+CurSt[1]+'"');
  End;
  CurSt:=Trim(CurSt);
End;
Procedure Prepare;
Begin
  Assign(Input,InFile);
  ReSet(Input);
  LineID:=0;
  NSource:=0;
  CurSt:='';
  Assign(Txt,OutFile);
  Assign(Prg,OutProg);
  TxtO:=False;
  PrgO:=False;
End;
Procedure ULexem;
Begin
  Error('Unexpected lexem: "'+LexName[CurLex]+'"');
End;
Procedure ScanCommands(Var Cmds:PCommand; FigureFinal:Boolean);
Var
  PP,PLast:PCommand;
Procedure UFault;
Begin
  PP:=Cmds;
  While PP<>Nil Do Begin
    If PP^.RefCmd<>Nil Then
      Cmds:=PP^.RefCmd
    Else
      Cmds:=PP^.Next;
    Dispose(PP);
    PP:=Cmds;
  End;
  ULexem;
End;
Procedure ScanCommand(Var PP:PCommand; IfPossible,ReturnPossible:Boolean);
Var
  I:LongInt;
  PPP:PCommand;
Begin
  Case CurLex Of
    _Identifier:Begin
      PP^.Data[0]:=CurCh;
      ScanLexem(False);
      If (CurLex<>_Operator) Or (CurCh<>'=') Then UFault;
      ScanLexem(True);
      Case CurLex Of
        _Number:Begin
          PP^.Valu[0]:=CurValue;
          PP^.Data[1]:=' ';
        End;
        _Identifier:Begin
          PP^.Data[1]:=CurCh;
        End;
        Else UFault;
      End;
      ScanLexem(False);
      Case CurLex Of
        _ORoundBracket:Begin
          PP^.Kind:=_Function;
          ScanLexem(True);
          I:=2;
          While CurLex<>_CRoundBracket Do Begin
            If CurLex=_Comma Then ScanLexem(True);
            Case CurLex Of
              _Number:Begin
                PP^.Valu[I]:=CurValue;
                PP^.Data[I]:=' ';
              End;
              _Identifier:Begin
                PP^.Data[I]:=CurCh;
              End;
              Else UFault;
            End;
            ScanLexem(False);
            Case CurLex Of
              _Comma:;
              _CRoundBracket:;
              Else UFault;
            End;
            Inc(I);
          End;
          ScanLexem(False);
        End;
        _Operator:Begin
          PP^.Kind:=_Operator;
          PP^.Data[2]:=CurCh;
          ScanLexem(True);
          Case CurLex Of
            _Number:Begin
              PP^.Valu[1]:=CurValue;
              PP^.Data[3]:=' ';
            End;
            _Identifier:Begin
              PP^.Data[3]:=CurCh;
            End;
            Else UFault;
          End;
          ScanLexem(False);
        End;
        Else Begin
          PP^.Kind:=_Identifier;
        End;
      End;
    End;
    _Print,_Return:Begin
      If (CurLex=_Return) And Not ReturnPossible Then UFault;
      PP^.Kind:=CurLex;
      ScanLexem(True);
      Case CurLex Of
        _Number:Begin
          PP^.Valu[0]:=CurValue;
          PP^.Data[1]:=' ';
        End;
        _Identifier:Begin
          PP^.Data[1]:=CurCh;
        End;
        Else UFault;
      End;
      ScanLexem(False);
    End;
    _If:If IfPossible Then Begin
      ScanLexem(False);
      If CurLex<>_Identifier Then UFault;
      PP^.Data[0]:=CurCh;
      ScanLexem(False);
      If CurLex<>_Then Then UFault;
      ScanLexem(False);
      New(PPP);
      FillChar(PPP^.Data,SizeOf(PPP^.Data),' ');
      FillChar(PPP^.Valu,SizeOf(PPP^.Valu),0);
      PPP^.Next:=PP^.Next;
      PPP^.LineNumber:=NSource;
      PP^.Next:=Nil;
      PPP^.RefCmd:=Nil;
      PP^.RefCmd:=PPP;
      PP^.Kind:=_If;
      ScanCommand(PPP,False,ReturnPossible);
    End Else UFault;
    Else UFault;
  End;
End;
Begin
  Cmds:=Nil;
  PLast:=Nil;
  Repeat
    New(PP);
    FillChar(PP^.Data,SizeOf(PP^.Data),' ');
    FillChar(PP^.Valu,SizeOf(PP^.Valu),0);
    PP^.RefCmd:=Nil;
    PP^.Next:=Nil;
    PP^.LineNumber:=NSource;
    If PLast=Nil Then Cmds:=PP Else PLast^.Next:=PP;
    ScanCommand(PP,True,FigureFinal);
    If PP^.Kind=_If Then PLast:=PP^.RefCmd Else PLast:=PP;
    Case CurLex Of
      _CFigureBracket:If FigureFinal Then Break Else UFault;
      _EOF:If FigureFinal Then UFault Else Break;
    End;
  Until False;
End;
Procedure ScanFunctions(Var Funcs:PFunction);
Var
  PP:PFunction;
  I:LongInt;
Procedure UFault;
Var
  PP:PFunction;
Begin
  PP:=Funcs;
  While PP<>Nil Do Begin
    Funcs:=PP^.Next;
    Dispose(PP);
    PP:=Funcs;
  End;
  ULexem;
End;
Begin
  Funcs:=Nil;
  While CurLex=_Function Do Begin
    New(PP);
    PP^.Next:=Funcs;
    Funcs:=PP;
    ScanLexem(False);
    If CurLex<>_Identifier Then UFault;
    PP^.Name:=CurCh;
    ScanLexem(False);
    If CurLex<>_ORoundBracket Then UFault;
    ScanLexem(False);
    I:=0;
    While CurLex<>_CRoundBracket Do Begin
      If CurLex=_Comma Then ScanLexem(False);
      If CurLex<>_Identifier Then UFault;
      PP^.ParNam[I]:=CurCh;
      ScanLexem(False);
      Case CurLex Of
        _Comma:;
        _CRoundBracket:;
        Else UFault;
      End;
      Inc(I);
    End;
    PP^.ParCnt:=I;
    ScanLexem(False);
    If CurLex<>_OFigureBracket Then UFault;
    ScanLexem(False);
    {Here memory might be lost if error}
    ScanCommands(PP^.FirstCmd,True);
    If CurLex<>_CFigureBracket Then UFault;
    ScanLexem(False);
  End;
End;
Procedure LoadProgram;
Begin
  ScanFunctions(Funcs);
  ScanCommands(Cmds,False);
End;
Function FindFunction(Name:Char):PFunction;
Var
  PP:PFunction;
Begin
  PP:=Funcs;
  While PP<>Nil Do Begin
    If PP^.Name=Name Then Begin FindFunction:=PP; Exit; End;
    PP:=PP^.Next;
  End;
  FindFunction:=Nil;
End;
Procedure ExecuteCommand(Var PP:PCommand);
Var
  I,Lf,Rt:LongInt;
  FPtr:PFunction;
Begin
  Case PP^.Kind Of
    _Identifier:Begin
      If PP^.Data[1]=' ' Then Vars[PP^.Data[0]]:=PP^.Valu[0] Else Vars[PP^.Data[0]]:=Vars[PP^.Data[1]];
      PP:=PP^.Next;
    End;
    _Operator:Begin
      If PP^.Data[1]=' ' Then Lf:=PP^.Valu[0] Else Lf:=Vars[PP^.Data[1]];
      If PP^.Data[3]=' ' Then Rt:=PP^.Valu[1] Else Rt:=Vars[PP^.Data[3]];
      Case PP^.Data[2] Of
        '+':Vars[PP^.Data[0]]:=Lf+Rt;
        '-':Vars[PP^.Data[0]]:=Lf-Rt;
        '*':Vars[PP^.Data[0]]:=Lf*Rt;
        '>':If Lf>Rt Then Vars[PP^.Data[0]]:=1 Else Vars[PP^.Data[0]]:=0;
        '=':If Lf=Rt Then Vars[PP^.Data[0]]:=1 Else Vars[PP^.Data[0]]:=0;
        '<':If Lf<Rt Then Vars[PP^.Data[0]]:=1 Else Vars[PP^.Data[0]]:=0;
      End;
      PP:=PP^.Next;
    End;
    _Function:Begin
      FPtr:=FindFunction(PP^.Data[1]);
      If FPtr=Nil Then Error('Unknown function: "'+PP^.Data[1]+'"');
      Stack[StkPos].RetPoint:=PP;
      Stack[StkPos].Vars:=Vars;
      FillChar(Vars,SizeOf(Vars),0);
      For I:=0 To FPtr^.ParCnt-1 Do Begin
        If PP^.Data[I+2]=' ' Then Lf:=PP^.Valu[I+2] Else Lf:=Stack[StkPos].Vars[PP^.Data[I+2]];
        Vars[FPtr^.ParNam[I]]:=Lf;
      End;
      Inc(StkPos);
      PP:=FPtr^.FirstCmd;
    End;
    _Print:Begin
      If Not NoPrint Then Begin
        If PP^.Data[1]=' ' Then Lf:=PP^.Valu[0] Else Lf:=Vars[PP^.Data[1]];
        Case Mode Of
          _Run,_Debug:WriteLn(Txt,Lf);
          _Compile:Begin
            Wrote[NWrote]:=Lf;
            Inc(NWrote);
          End;
        End;
      End;
      PP:=PP^.Next;
    End;
    _Return:Begin
      If PP^.Data[1]=' ' Then Lf:=PP^.Valu[0] Else Lf:=Vars[PP^.Data[1]];
      Dec(StkPos);
      PP:=Stack[StkPos].RetPoint;
      Vars:=Stack[StkPos].Vars;
      Vars[PP^.Data[0]]:=Lf;
      PP:=PP^.Next;
    End;
    _If:Begin
      If Vars[PP^.Data[0]]<>0 Then Begin
        PP:=PP^.RefCmd;
        ExecuteCommand(PP);
      End Else PP:=PP^.RefCmd^.Next;
    End;
  End;
End;
Procedure ApplyRun;
Var
  PP:PCommand;
  {Numb:LongInt;}
Begin
  LoadProgram;
  StkPos:=0;
  PP:=Cmds;
  FillChar(Vars,SizeOf(Vars),0);
  {Numb:=0;}
  While PP<>Nil Do Begin
    ExecuteCommand(PP);
    {Inc(NUmb);}
  End;
  {WriteLn(Numb);}
  FreeAll;
End;
Procedure ApplyCompile;
Const
  Header:Array[0..1]Of String=(
    'Program TextWriter;',
    'Const');
  Middle:Array[0..0] Of String=('  ToWrite:Array[1..N]Of LongInt=(');
  Footer:Array[0..8]Of String=(
    'Var',
    '  I:LongInt;',
    'Begin',
    '  Assign(Output,''OUTPUT.TXT'');',
    '  ReWrite(Output);',
    '  For I:=1 To N Do',
    '    WriteLn(ToWrite[I]);',
    '  Close(Output);',
    'End.');
Var
  I:LongInt;
Begin
  NWrote:=0;
  ApplyRun;
  For I:=Low(Header) To High(Header) Do WriteLn(Prg,Header[I]);
  WriteLn(Prg,'  N=',NWrote,';');
  For I:=Low(Middle) To High(Middle) Do WriteLn(Prg,Middle[I]);
  For I:=0 To NWrote-1 Do Begin
    Write(Prg,'    ',Wrote[I]);
    If I=(NWrote-1) Then WriteLn(Prg,');') Else WriteLn(Prg,',');
  End;
  For I:=Low(Footer) To High(Footer) Do WriteLn(Prg,Footer[I]);
End;
Procedure ApplyDebug;
Var
  PP:PCommand;
  LeftX:LongInt;
  OldV:TVars;
  Chg:Array['a'..'z']Of Boolean;
  Ch:Char;
  SaveSt:LongInt;
Procedure RedrawText;
Var
  I:LongInt;
  Start,Finish:LongInt;
  St:String;
Begin
  GotoXY(1,2);
  If PP=Nil Then Begin
    Finish:=NSource;
    Start:=Finish-22;
  End Else Begin
    Start:=PP^.LineNumber-11;
    If (Start+22)>NSource Then Start:=NSource-22;
  End;
  If Start<1 Then Start:=1;
  Finish:=Start+22;
  For I:=Start To Finish Do Begin
    If I<=NSource Then
      St:=Copy(Source[I],LeftX,49)
    Else
      St:='';
    If (PP<>Nil) And (I=PP^.LineNumber) Then Begin
      TextColor(14);
      TextBackGround(3);
    End Else Begin
      TextColor(14);
      TextBackGround(1);
    End;
    While Length(St)<49 Do St:=St+' ';
    Write(St);
    TextColor(14);
    TextBackGround(1);
    St:='';
    St:=St+'';
    While Length(St)<31 Do St:=St+' ';
    Write(St);
  End;
End;
Procedure WriteWatches;
Var
  I:LongInt;
  Used:Boolean;
  Ch:Char;
  SaveV:TVars;
  SaveSt:LongInt;
  PPSave:PCommand;
Begin
  For Ch:='a' To 'z' Do Begin
    I:=Ord(Ch)-Ord('a');
    If Not Odd(I) Then GotoXY(52,I Div 2+2) Else GotoXY(67,I Div 2+2);
    Used:=False;
    If PP<>Nil Then
      For I:=0 To MaxData-1 Do If PP^.Data[I]=Ch Then Used:=True;
    If (PP<>Nil) And (PP^.RefCmd<>Nil) Then
      For I:=0 To MaxData-1 Do If PP^.RefCmd^.Data[I]=Ch Then Used:=True;
    If Chg[Ch] Then
      If Used Then
        TextColor(14)
      Else
        TextColor(12)
    Else
      If Used Then
        TextColor(10)
      Else
        TextColor(15);
    Write(Ch,' = ',Vars[Ch]);
  End;
  TextColor(15);
  If PP=Nil Then Begin
    GotoXY(52,16);
    Write('Program finished');
    GotoXY(52,17);
    Write('Press any key to exit');
  End Else Case PP^.Kind Of
    _Identifier,_Operator,_Function:Begin
      Ch:=PP^.Data[0];
      GotoXY(52,16);
      Write('Old value of '+Ch+': ',Vars[Ch]);
      SaveV:=Vars;
      SaveSt:=StkPos;
      NoPrint:=True;
      PPSave:=PP;
      Repeat
        ExecuteCommand(PP);
      Until (PP=Nil) Or (StkPos<=SaveSt);
      NoPrint:=False;
      GotoXY(52,17);
      Write('New value of '+ch+': ',Vars[Ch]);
      PP:=PPSave;
      Vars:=SaveV;
      StkPos:=SaveSt;
    End;
    _If:Begin
      Ch:=PP^.Data[0];
      GotoXY(52,16);
      If Vars[Ch]<>0 Then Write('"Then" will be called') Else Write('"Then" won''t be called');
      GotoXY(52,17);
      Write('(because ',Ch,' = ',Vars[Ch],')');
      If Vars[Ch]<>0 Then Case PP^.RefCmd^.Kind Of
        _Identifier,_Operator,_Function:Begin
          Ch:=PP^.RefCmd^.Data[0];
          GotoXY(52,19);
          Write('Old value of '+Ch+': ',Vars[Ch]);
          SaveV:=Vars;
          SaveSt:=StkPos;
          NoPrint:=True;
          PPSave:=PP;
          Repeat
            ExecuteCommand(PP);
          Until (PP=Nil) Or (StkPos<=SaveSt);
          NoPrint:=False;
          GotoXY(52,20);
          Write('New value of '+Ch+': ',Vars[Ch]);
          PP:=PPSave;
          Vars:=SaveV;
          StkPos:=SaveSt;
        End;
        _Print:Begin
          If PP^.RefCmd^.Data[1]=' ' Then I:=PP^.RefCmd^.Valu[0] Else I:=Vars[PP^.RefCmd^.Data[1]];
          GotoXY(52,19);
          Write(I,' will be printed');
        End;
      End;
    End;
    _Print:Begin
      If PP^.Data[1]=' ' Then I:=PP^.Valu[0] Else I:=Vars[PP^.Data[1]];
      GotoXY(52,16);
      Write(I,' will be printed');
    End;
  End;
End;
Begin
  LoadProgram;
  StkPos:=0;
  FillChar(Vars,SizeOf(Vars),0);
  PP:=Cmds;
  TextColor(0);
  TextBackGround(7);
  CursorOff;
  ClrScr;
  Write('-------------- Free-VMAG debugger v1.0 (c) 2001 by Mitrichev Peter -------------');
  GotoXY(1,25);
  Write(' F1 - help, F7 - trace, F8 - step, Ctrl+F2 - exit, F9 - run, F10 - till return ');
  Window(1,2,80,24);
  TextColor(14);
  TextBackGround(1);
  ClrScr;
  Window(1,1,80,25);
  LeftX:=1;
  FillChar(Chg,SizeOf(Chg),False);
  Repeat
    RedrawText;
    WriteWatches;
    Case ReadKey Of
      #0:Case ReadKey Of
        #75:If LeftX>1 Then Dec(LeftX); {Left}
        #77:Inc(LeftX); {Right}
        #71:LeftX:=1; {Home}
        #95:Break; {Ctrl+F2}
        #65:Begin {F7}
          OldV:=Vars;
          ExecuteCommand(PP);
          For Ch:='a' To 'z' Do Chg[Ch]:=OldV[Ch]<>Vars[Ch];
          If PP=Nil Then Begin
            RedrawText;
            WriteWatches;
            If ReadKey='' Then;
            Break;
          End;
        End;
        #66:Begin {F8}
          OldV:=Vars;
          SaveSt:=StkPos;
          Repeat
            ExecuteCommand(PP);
          Until (PP=Nil) Or (StkPos<=SaveSt);
          For Ch:='a' To 'z' Do Chg[Ch]:=OldV[Ch]<>Vars[Ch];
          If PP=Nil Then Begin
            RedrawText;
            WriteWatches;
            If ReadKey='' Then;
            Break;
          End;
        End;
        #67:Begin {F9}
          OldV:=Vars;
          Repeat
            ExecuteCommand(PP);
          Until (PP=Nil);
          For Ch:='a' To 'z' Do Chg[Ch]:=OldV[Ch]<>Vars[Ch];
          If PP=Nil Then Begin
            RedrawText;
            WriteWatches;
            If ReadKey='' Then;
            Break;
          End;
        End;
        #68:Begin {F10}
          OldV:=Vars;
          SaveSt:=StkPos;
          Repeat
            ExecuteCommand(PP);
          Until (PP=Nil) Or (StkPos<SaveSt);
          For Ch:='a' To 'z' Do Chg[Ch]:=OldV[Ch]<>Vars[Ch];
          If PP=Nil Then Begin
            RedrawText;
            WriteWatches;
            If ReadKey='' Then;
            Break;
          End;
        End;
        #59:Begin {F1}
          HelpScreen;
          If ReadKey='' Then;
          While KeyPressed Do ReadKey;
        End;
      End;
    End;
  Until False;
  TextColor(7);
  TextBackGround(0);
  ClrScr;
  While KeyPressed Do ReadKey;
  FreeAll;
End;
Procedure Solve;
Begin
  NoPrint:=False;
  ScanLexem(False);
  Case CurLex Of
    _Run:Mode:=_Run;
    _Compile:Mode:=_Compile;
    _Debug:Mode:=_Debug;
    Else ULexem;
  End;
  ScanLexem(False);
  If Mode=_Compile Then Begin
    Rewrite(Prg);
    PrgO:=True;
    ApplyCompile;
    Exit;
  End;
  Rewrite(Txt);
  TxtO:=True;
  Case Mode Of
    _Run:ApplyRun;
    _Debug:ApplyDebug;
  End;
End;
Begin
  Prepare;
  Solve;
  CloseAll;
End.
