{$I-,Q-,R-,S-,V-,X-}
type  float = extended;
const eps: float = 1e-10;
var
  x, y, z, t: array [1..4] of float;
  xx, yy, zz, tt: float;
  m: array [1..4, 1..5] of float;
  mx: array [1..4] of integer;
  ne: integer;

procedure makeeq(e, i, j: integer);
begin
  m[e][1] := 2*(x[j]-x[i]);
  m[e][2] := 2*(y[j]-y[i]);
  m[e][3] := 2*(z[j]-z[i]);
  m[e][4] := 2*(t[j]-t[i]);
  m[e][5] := sqr(x[j]) - sqr(x[i]) + sqr(y[j]) - sqr(y[i]) +
             sqr(z[j]) - sqr(z[i]) + sqr(t[i]) - sqr(t[j]);
end;

procedure swp(var a, b: float);
var  h: float;
begin
  h := a; a := b; b := h;
end;

procedure swpr(i, j: integer);
var  k: integer;
begin
  for k := 1 to 5 do swp(m[i][k], m[j][k]);
end;

procedure mutII(r, w: integer; const a: float);
var  k: integer;
begin
  for k := 1 to 5 do m[r][k] := m[r][k] + (a*(m[w][k]));
end;

procedure gauss;
var
  i, j, k, e: integer;
begin
  e := 0;
  for i := 1 to 4 do begin
    j := e+1;
    while ((j<=4) and (abs(m[j][i])<eps)) do inc(j);
    if (j<=4) then begin
      inc(e);
      swpr(e, j);
      mx[e] := i;
      for k := i+1 to 4 do mutII(k, e, -((m[k][i])/(m[e][i])));
    end;
  end;
  ne := e;
end;

procedure mksol;
var
  i, j, e: integer;
  s, aa, bb, cc, l, d: float;
  a, b: array [1..4] of float;
begin
  e := ne;
  a[4] := 0; b[4] := 1;
  for i := 3 downto 1 do begin
    if (mx[e]=i) then begin
      a[i] := m[e][5]; b[i] := 0;
      for j := i+1 to 4 do begin
        a[i] := a[i] - (a[j]*m[e][j]);
        b[i] := b[i] - (b[j]*m[e][j]);
      end;
      a[i] := a[i]/(m[e][i]);
      b[i] := b[i]/(m[e][i]);
      dec(e);
    end else begin a[i] := 0; b[i] := 0; end;
  end;

  aa := sqr(b[1]) + sqr(b[2]) + sqr(b[3]) - 1;
  bb := 2*((b[1]*(a[1]-x[1]))+(b[2]*(a[2]-y[1]))+(b[3]*(a[3]-z[1]))-(t[1]));
  cc := sqr(a[1]-x[1])+sqr(a[2]-y[1])+sqr(a[3]-z[1])-sqr(t[1]);

  l := 1;
  if (abs(aa)>eps) then begin
    d := abs(sqr(bb) - (4*aa*cc));
    l := ((-bb) + sqrt(d))/(2*aa);
    s := ((-bb) - sqrt(d))/(2*aa);
    if (s>l) then l := s;
  end else begin
    if (abs(bb)>eps) then l := -(cc/bb);
  end;

  xx := a[1] + (b[1]*l);
  yy := a[2] + (b[2]*l);
  zz := a[3] + (b[3]*l);
end;

procedure Solve;
var
  i: integer;
begin
  makeeq(1, 1, 2); makeeq(2, 2, 3); makeeq(3, 3, 4); makeeq(4, 4, 1);

  gauss;
  mksol;
end;

procedure ReadAll;
var i: integer;
begin
  Assign(Input, 'INPUT.TXT'); Reset(Input);

  for i := 1 to 4 do Read(x[i], y[i], z[i]);
  t[1] := 0;
  for i := 2 to 4 do Read(t[i]);

  Close(Input);
end;

procedure Out;
begin
  Assign(Output, 'OUTPUT.TXT'); ReWrite(Output);

  WriteLn(xx:1:4, ' ', yy:1:4, ' ', zz:1:4);

  Close(Output);
end;

begin
  ReadAll;
  Solve;
  Out;
end.
